@@ -253,7 +253,7 @@ search_find(
 	#if  (GTK_MAJOR_VERSION >= 2) && (GTK_MINOR_VERSION >= 14)
 		gtk_adjustment_set_value( terminal->adjustment , value) ;
 	#else
-		VTE_WIDGET(screen)->adjustment->value = value ;
+		terminal->adjustment->value = value ;
 		gtk_adjustment_value_changed( terminal->adjustment) ;
 	#endif
 
@@ -1312,8 +1312,14 @@ vte_terminal_realize(
 
 	xid = gdk_x11_drawable_get_xid( widget->window) ;
 
-	if( disp.visual == DefaultVisual( disp.display , disp.screen))
+	if( disp.gc->gc == DefaultGC( disp.display , disp.screen))
 	{
+		/*
+		 * Replace visual, colormap, depth and gc with those inherited from parent xid.
+		 * In some cases that those of parent xid is not DefaultVisual, DefaultColormap
+		 * and so on (e.g. compiz), BadMatch error can happen.
+		 */
+
 		XWindowAttributes  attr ;
 		XGCValues  gc_value ;
 
@@ -1322,7 +1328,7 @@ vte_terminal_realize(
 		disp.colormap = attr.colormap ;
 		disp.depth = attr.depth ;
 
-		/* x_gc_t using DefaultGC is already created in vte_terminal_init */
+		/* x_gc_t using DefaultGC is already created in vte_terminal_class_init */
 		gc_value.background = disp.gc->bg_color ;
 		gc_value.graphics_exposures = 0 ;
 		disp.gc->gc = XCreateGC( disp.display , xid ,
@@ -1573,11 +1579,10 @@ vte_terminal_class_init(
 	x_prepare_for_main_config( conf) ;
 	x_main_config_init( &main_config , conf , 1 , argv) ;
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	if( main_config.type_engine == TYPE_XCORE)
 	{
 		/*
-		 * Hack
+		 * XXX Hack
 		 * Default value of type_engine is TYPE_XCORE in normal mlterm,
 		 * but default value in libvte compatible library of mlterm is TYPE_XFT.
 		 */
@@ -1586,14 +1591,18 @@ vte_terminal_class_init(
 		if( ( value = kik_conf_get_value( conf , "type_engine")) == NULL ||
 			strcmp( value , "xcore") != 0)
 		{
-		#ifdef  USE_TYPE_XFT
+			/*
+			 * cairo is prefered if mlterm works as libvte because gtk+
+			 * usually depends on cairo.
+			 */
+		#if  ! defined(USE_TYPE_CAIRO) && defined(USE_TYPE_XFT)
 			main_config.type_engine = TYPE_XFT ;
 		#else
 			main_config.type_engine = TYPE_CAIRO ;
 		#endif
 		}
 	}
-#endif
+
 	/* Default value of vte "audible-bell" is TRUE, while "visible-bell" is FALSE. */
 	main_config.bel_mode = BEL_SOUND ;
 
@@ -1961,7 +1970,7 @@ vte_terminal_init(
 
 	/* related to x_font_use_point_size_for_fc(1) in vte_terminal_class_init. */
 	if( ( dpi = gdk_screen_get_resolution(
-		gtk_widget_get_screen( GTK_WIDGET(terminal)))) != -1)
+			gtk_widget_get_screen( GTK_WIDGET(terminal)))) != -1)
 	{
 	#ifdef  DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " Setting dpi %f\n" , dpi) ;
@@ -2071,6 +2080,9 @@ vte_terminal_init(
 	reset_vte_size_member( terminal) ;
 }
 
+/*
+ * "WINDOWID" is set by terminal applications using vte.
+ */
 #if  0
 static char **
 modify_envv(
@@ -2085,7 +2097,7 @@ modify_envv(
 	p = envv ;
 	while( *p)
 	{
-		if( **p && strncmp( *p , "WINDOWID" , K_MIN(strlen(*p),8)) == 0)
+		if( **p && strncmp( *p , "WINDOWID" , K_MIN(strlen(*p) , 8)) == 0)
 		{
 			char *  wid_env ;
 
@@ -2344,7 +2356,7 @@ vte_terminal_paste_clipboard(
 	VteTerminal *  terminal
 	)
 {
-	x_screen_set_config( terminal->pvt->screen , NULL , "paste" , NULL) ;
+	x_screen_exec_cmd( terminal->pvt->screen , "paste") ;
 }
 
 void
@@ -2747,13 +2759,12 @@ vte_terminal_set_background_image_file(
 	kik_debug_printf( KIK_DEBUG_TAG " Setting image file %s\n" , path) ;
 #endif
 
-	if( terminal->pvt->image &&
-	    /*
-	     * Don't unref terminal->pvt->image if path is
-	     * "pixmap:<pixmap id>" (Ex. the case of vte_terminal_set_background_image_file()
-	     * being called from update_wall_picture().)
-	     */
-	    (strlen(path) <= 7 || strncmp( path , "pixmap:" , 7) != 0))
+	/*
+	 * Don't unref terminal->pvt->image if path is
+	 * "pixmap:<pixmap id>" (Ex. the case of vte_terminal_set_background_image_file()
+	 * being called from update_wall_picture().)
+	 */
+	if( terminal->pvt->image && strncmp( path , "pixmap:" , 7) != 0)
 	{
 		g_object_unref( terminal->pvt->image) ;
 		terminal->pvt->image = NULL ;
@@ -3120,7 +3131,7 @@ vte_terminal_reset(
 {
 	if( GTK_WIDGET_REALIZED(GTK_WIDGET(terminal)))
 	{
-		x_screen_set_config( terminal->pvt->screen , NULL , "full_reset" , NULL) ;
+		x_screen_exec_cmd( terminal->pvt->screen , "full_reset") ;
 	}
 }
 
