@@ -12,7 +12,6 @@
 
 #include  "ml_edit_util.h"
 #include  "ml_edit_scroll.h"
-#include  "ml_bidi.h"
 
 
 #if  0
@@ -527,7 +526,7 @@ ml_edit_overwrite_chars(
 				break ;
 			}
 
-			ml_line_set_continued_to_next( line) ;
+			ml_line_set_continued_to_next( line , 1) ;
 
 			edit->cursor.char_index = edit->cursor.col = 0 ;
 			if( edit->cursor.row + 1 > edit->scroll_region_end)
@@ -592,7 +591,7 @@ ml_edit_overwrite_chars(
 
 	ml_line_overwrite( line , edit->cursor.char_index , &buffer[beg] ,
 		count - beg , cols) ;
-	ml_line_unset_continued_to_next( line) ;
+	ml_line_set_continued_to_next( line , 0) ;
 
 	ml_cursor_moveh_by_char( &edit->cursor , new_char_index) ;
 
@@ -1168,22 +1167,6 @@ ml_edit_set_modified_all(
 	return  1 ;
 }
 
-u_int
-ml_edit_get_cols(
-	ml_edit_t *  edit
-	)
-{
-	return  edit->model.num_of_cols ;
-}
-
-u_int
-ml_edit_get_rows(
-	ml_edit_t *  edit
-	)
-{
-	return  edit->model.num_of_rows ;
-}
-
 int
 ml_edit_goto_beg_of_line(
 	ml_edit_t *  edit
@@ -1403,8 +1386,8 @@ ml_edit_go_downward(
 		if( ! ( flag & SCROLL))
 		{
 		#ifdef  DEBUG
-			kik_warn_printf(
-				KIK_DEBUG_TAG " cursor cannot go downward(reaches scroll lower limit).\n") ;
+			kik_warn_printf( KIK_DEBUG_TAG
+				" cursor cannot go downward(reaches scroll lower limit).\n") ;
 		#endif
 
 			return  0 ;
@@ -1457,9 +1440,6 @@ ml_edit_set_relative_origin(
 {
 	edit->is_relative_origin = 1 ;
 
-	/* cursor position is reset(the same behavior of xterm 4.0.3, kterm 6.2.0 or so) */
-	ml_edit_goto( edit , 0 , 0) ;
-
 	return  1 ;
 }
 
@@ -1470,30 +1450,36 @@ ml_edit_set_absolute_origin(
 {
 	edit->is_relative_origin = 0 ;
 
-	/* cursor position is reset(the same behavior of xterm 4.0.3, kterm 6.2.0 or so) */
-	ml_edit_goto( edit , 0 , 0) ;
-
 	return  1 ;
 }
 
 int
 ml_edit_set_auto_wrap(
-	ml_edit_t *  edit
+	ml_edit_t *  edit ,
+	int  flag
 	)
 {
-	edit->is_auto_wrap = 1 ;
+	edit->is_auto_wrap = flag ;
 
 	return  1 ;
 }
 
 int
-ml_edit_unset_auto_wrap(
-	ml_edit_t *  edit
+ml_edit_set_bce_fg_color(
+	ml_edit_t *  edit ,
+	ml_color_t  fg_color
 	)
 {
-	edit->is_auto_wrap = 0 ;
+	return  ml_char_set_fg_color( &edit->bce_ch , fg_color) ;
+}
 
-	return  1 ;
+int
+ml_edit_set_bce_bg_color(
+	ml_edit_t *  edit ,
+	ml_color_t  bg_color
+	)
+{
+	return  ml_char_set_bg_color( &edit->bce_ch , bg_color) ;
 }
 
 int
@@ -1512,29 +1498,6 @@ ml_edit_restore_cursor(
 	return  ml_cursor_restore( &edit->cursor) ;
 }
 
-int
-ml_cursor_char_index(
-	ml_edit_t *  edit
-	)
-{
-	return  edit->cursor.char_index ;
-}
-
-int
-ml_cursor_col(
-	ml_edit_t *  edit
-	)
-{
-	return  edit->cursor.col ;
-}
-
-int
-ml_cursor_row(
-	ml_edit_t *  edit
-	)
-{
-	return  edit->cursor.row ;
-}
 
 /*
  * for debugging.
