@@ -38,8 +38,8 @@ get_n_prev_char_pos(
 {
 	int  count ;
 
-	*char_index = screen->edit->cursor.char_index ;
-	*row = screen->edit->cursor.row ;
+	*char_index = ml_cursor_char_index( screen->edit) ;
+	*row = ml_cursor_row( screen->edit) ;
 
 	for( count = 0 ; count < n ; count ++)
 	{
@@ -392,7 +392,7 @@ reverse_or_restore_color(
 
 static u_int
 check_or_copy_region(
-	ml_screen_t *  screen ,
+	ml_screen_t *  screen ,		/* visual */
 	ml_char_t *  chars ,
 	u_int  num_of_chars ,
         int  beg_char_index ,		/* can be over size_except_spaces */
@@ -472,22 +472,18 @@ check_or_copy_region(
 
 		break ;
 	}
-		
+
 	if( row == end_row)
 	{
-		if( end_char_index < 0)
-		{
-			end_char_index = beg_regarding_rtl ;
-		}
-		else if( end_char_index >= size_except_spaces)
-		{
-			end_char_index = K_MAX(size_except_spaces,1) - 1 ;
-		}
-		
 		if( ml_line_is_rtl( line))
 		{
-			count = K_MIN(beg_char_index + 1,size_except_spaces) -
-					K_MAX(end_char_index,beg_regarding_rtl) ;
+			if( end_char_index < beg_regarding_rtl)
+			{
+				end_char_index = beg_regarding_rtl ;
+			}
+
+			count = K_MIN(beg_char_index + 1,size_except_spaces) - end_char_index ;
+
 			if( chars)
 			{
 				ml_line_copy_logical_str( line , chars , end_char_index , count) ;
@@ -495,8 +491,13 @@ check_or_copy_region(
 		}
 		else
 		{
-			count = K_MIN(end_char_index + 1,size_except_spaces) -
-					K_MAX(beg_char_index,beg_regarding_rtl) ;
+			if( end_char_index >= size_except_spaces)
+			{
+				end_char_index = K_MAX(size_except_spaces,1) - 1 ;
+			}
+
+			count = end_char_index + 1 - beg_char_index ;
+
 			if( chars)
 			{
 				ml_line_copy_logical_str( line , chars , beg_char_index , count) ;
@@ -778,48 +779,6 @@ ml_screen_resize(
 	return  1 ;
 }
 
-int
-ml_screen_set_bce_fg_color(
-	ml_screen_t *  screen ,
-	ml_color_t  fg_color
-	)
-{
-	return  ml_char_set_fg_color( &screen->edit->bce_ch , fg_color) ;
-}
-
-int
-ml_screen_set_bce_bg_color(
-	ml_screen_t *  screen ,
-	ml_color_t  bg_color
-	)
-{
-	return  ml_char_set_bg_color( &screen->edit->bce_ch , bg_color) ;
-}
-
-int
-ml_screen_cursor_col(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_cursor_col( screen->edit) ;
-}
-
-int
-ml_screen_cursor_char_index(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_cursor_char_index( screen->edit) ;
-}
-
-int
-ml_screen_cursor_row(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_cursor_row( screen->edit) ;
-}
-
 int
 ml_screen_cursor_row_in_screen(
 	ml_screen_t *  screen
@@ -840,22 +799,6 @@ ml_screen_cursor_row_in_screen(
 	return  row ;
 }
 
-u_int
-ml_screen_get_cols(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_get_cols( screen->edit) ;
-}
-
-u_int
-ml_screen_get_rows(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_get_rows( screen->edit) ;
-}
-
 u_int
 ml_screen_get_logical_cols(
 	ml_screen_t *  screen
@@ -886,31 +829,6 @@ ml_screen_get_logical_rows(
 	}
 }
 
-u_int
-ml_screen_get_log_size(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_get_log_size( &screen->logs) ;
-}
-
-u_int
-ml_screen_change_log_size(
-	ml_screen_t *  screen ,
-	u_int  log_size
-	)
-{
-	return  ml_change_log_size( &screen->logs , log_size) ;
-}
-
-u_int
-ml_screen_get_num_of_logged_lines(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_get_num_of_logged_lines( &screen->logs) ;
-}
-
 int
 ml_screen_convert_scr_row_to_abs(
 	ml_screen_t *  screen ,
@@ -1313,23 +1231,6 @@ ml_screen_backscroll_downward(
 	return  1 ;
 }
 
-u_int
-ml_screen_get_tab_size(
-	ml_screen_t *  screen
-	)
-{
-	return  screen->edit->tab_size ;
-}
-
-int
-ml_screen_set_tab_size(
-	ml_screen_t *  screen ,
-	u_int  tab_size
-	)
-{
-	return  ml_edit_set_tab_size( screen->edit , tab_size) ;
-}
-
 int
 ml_screen_reverse_color(
 	ml_screen_t *  screen ,
@@ -1340,7 +1241,7 @@ ml_screen_reverse_color(
 	)
 {
 	return  reverse_or_restore_color( screen , beg_char_index , beg_row ,
-		end_char_index , end_row , ml_line_reverse_color) ;
+			end_char_index , end_row , ml_line_reverse_color) ;
 }
 
 int
@@ -1353,7 +1254,7 @@ ml_screen_restore_color(
 	)
 {
 	return  reverse_or_restore_color( screen , beg_char_index , beg_row ,
-		end_char_index , end_row , ml_line_restore_color) ;
+			end_char_index , end_row , ml_line_restore_color) ;
 }
 
 u_int
@@ -1664,57 +1565,6 @@ ml_screen_insert_chars(
 	return  ml_edit_insert_chars( screen->edit , chars , len) ;
 }
 
-int
-ml_screen_insert_blank_chars(
-	ml_screen_t *  screen ,
-	u_int  len
-	)
-{
-	return  ml_edit_insert_blank_chars( screen->edit , len) ;
-}
-
-int
-ml_screen_vertical_forward_tabs(
-	ml_screen_t *  screen ,
-	u_int  num
-	)
-{
-	return  ml_edit_vertical_forward_tabs( screen->edit , num) ;
-}
-
-int
-ml_screen_vertical_backward_tabs(
-	ml_screen_t *  screen ,
-	u_int  num
-	)
-{
-	return  ml_edit_vertical_backward_tabs( screen->edit , num) ;
-}
-
-int
-ml_screen_set_tab_stop(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_set_tab_stop( screen->edit) ;
-}
-
-int
-ml_screen_clear_tab_stop(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_clear_tab_stop( screen->edit) ;
-}
-
-int
-ml_screen_clear_all_tab_stops(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_clear_all_tab_stops( screen->edit) ;
-}
-
 int
 ml_screen_insert_new_lines(
 	ml_screen_t *  screen ,
@@ -1731,14 +1581,6 @@ ml_screen_insert_new_lines(
 	return  1 ;
 }
 
-int
-ml_screen_line_feed(
-	ml_screen_t *  screen
-	)
-{	
-	return  ml_edit_go_downward( screen->edit , SCROLL) ;
-}
-
 int
 ml_screen_overwrite_chars(
 	ml_screen_t *  screen ,
@@ -1749,24 +1591,6 @@ ml_screen_overwrite_chars(
 	return  ml_edit_overwrite_chars( screen->edit , chars , len) ;
 }
 
-int
-ml_screen_delete_cols(
-	ml_screen_t *  screen ,
-	u_int  len
-	)
-{
-	return  ml_edit_delete_cols( screen->edit , len) ;
-}
-
-int
-ml_screen_clear_cols(
-	ml_screen_t *  screen ,
-	u_int  cols
-	)
-{
-	return  ml_edit_clear_cols( screen->edit , cols) ;
-}
-
 int
 ml_screen_delete_lines(
 	ml_screen_t *  screen ,
@@ -1790,82 +1614,6 @@ ml_screen_delete_lines(
 	return  1 ;
 }
 
-int
-ml_screen_clear_line_to_right(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_clear_line_to_right( screen->edit) ;
-}
-
-int
-ml_screen_clear_line_to_left(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_clear_line_to_left( screen->edit) ;
-}
-
-int
-ml_screen_clear_below(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_clear_below( screen->edit) ;
-}
-
-int
-ml_screen_clear_above(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_clear_above( screen->edit) ;
-}
-
-int
-ml_screen_set_scroll_region(
-	ml_screen_t *  screen ,
-	int  beg ,
-	int  end
-	)
-{
-	return  ml_edit_set_scroll_region( screen->edit , beg , end) ;
-}
-
-int
-ml_screen_index(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_go_downward( screen->edit , SCROLL) ;
-}
-
-int
-ml_screen_reverse_index(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_go_upward( screen->edit , SCROLL) ;
-}
-
-int
-ml_screen_scroll_upward(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	return  ml_edit_scroll_upward( screen->edit , size) ;
-}
-
-int
-ml_screen_scroll_downward(
-	ml_screen_t *  screen ,
-	u_int  size
-	)
-{
-	return  ml_edit_scroll_downward( screen->edit , size) ;
-}
-
 int
 ml_screen_go_forward(
 	ml_screen_t *  screen ,
@@ -1958,14 +1706,6 @@ ml_screen_go_downward(
 	return  1 ;
 }
 
-int
-ml_screen_goto_beg_of_line(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_goto_beg_of_line( screen->edit) ;
-}
-
 int
 ml_screen_go_horizontally(
 	ml_screen_t *  screen ,
@@ -1984,72 +1724,6 @@ ml_screen_go_vertically(
 	return  ml_screen_goto( screen , ml_cursor_col( screen->edit) , row) ;
 }
 
-int
-ml_screen_goto_home(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_goto_home( screen->edit) ;
-}
-
-int
-ml_screen_goto(
-	ml_screen_t *  screen ,
-	int  col ,
-	int  row
-	)
-{
-	return  ml_edit_goto( screen->edit , col , row) ;
-}
-
-int
-ml_screen_set_relative_origin(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_set_relative_origin( screen->edit) ;
-}
-
-int
-ml_screen_set_absolute_origin(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_set_absolute_origin( screen->edit) ;
-}
-
-int
-ml_screen_set_auto_wrap(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_set_auto_wrap( screen->edit) ;
-}
-
-int
-ml_screen_unset_auto_wrap(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_unset_auto_wrap( screen->edit) ;
-}
-
-int
-ml_screen_save_cursor(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_save_cursor( screen->edit) ;
-}
-
-int
-ml_screen_restore_cursor(
-	ml_screen_t *  screen
-	)
-{
-	return  ml_edit_restore_cursor( screen->edit) ;
-}
-
 int
 ml_screen_cursor_visible(
 	ml_screen_t *  screen
@@ -2080,14 +1754,6 @@ ml_screen_cursor_invisible(
 	return  1 ;
 }
 
-int
-ml_screen_is_cursor_visible(
-	ml_screen_t *  screen
-	)
-{
-	return screen->is_cursor_visible ;
-}
-
 int
 ml_screen_is_alternative_edit(
 	ml_screen_t *  screen
@@ -2110,7 +1776,8 @@ ml_screen_use_normal_edit(
 
 	if( screen->logvis)
 	{
-		(*screen->logvis->init)( screen->logvis , &screen->edit->model , &screen->edit->cursor) ;
+		(*screen->logvis->init)( screen->logvis ,
+			&screen->edit->model , &screen->edit->cursor) ;
 	}
 
 	ml_edit_set_modified_all( screen->edit) ;
@@ -2130,7 +1797,8 @@ ml_screen_use_alternative_edit(
 
 	if( screen->logvis)
 	{
-		(*screen->logvis->init)( screen->logvis , &screen->edit->model , &screen->edit->cursor) ;
+		(*screen->logvis->init)( screen->logvis ,
+			&screen->edit->model , &screen->edit->cursor) ;
 	}
 
 	ml_edit_set_modified_all( screen->edit) ;
