@@ -92,10 +92,6 @@ ml_term_new(
 	term->icon_name = NULL ;
 	term->icon_path = NULL ;
 
-	term->is_mouse_pos_sending = 0 ;
-	term->is_app_keypad = 0 ;
-	term->is_app_cursor_keys = 0 ;
-
 	term->is_attached = 0 ;
 
 	return  term ;
@@ -268,14 +264,6 @@ ml_term_detach(
 	return  1 ;
 }
 
-int
-ml_term_is_attached(
-	ml_term_t *  term
-	)
-{
-	return  term->is_attached ;
-}
-
 int
 ml_term_set_auto_encoding(
 	ml_term_t *  term ,
@@ -288,21 +276,57 @@ ml_term_set_auto_encoding(
 }
 
 int
-ml_term_is_auto_encoding(
-	ml_term_t *  term
-	)
-{
-	return  term->is_auto_encoding ;
-}
-
-int
-ml_term_set_logging_vt_seq(
+ml_term_set_use_bidi(
 	ml_term_t *  term ,
 	int  flag
 	)
 {
-	term->parser->logging_vt_seq = flag ;
-	
+	term->use_bidi = flag ;
+
+	return  1 ;
+}
+
+int
+ml_term_set_bidi_mode(
+	ml_term_t *  term ,
+	ml_bidi_mode_t  mode
+	)
+{
+	term->bidi_mode = mode ;
+
+	return  1 ;
+}
+
+int
+ml_term_set_use_ind(
+	ml_term_t *  term ,
+	int  flag
+	)
+{
+	term->use_ind = flag ;
+
+	return  1 ;
+}
+
+int
+ml_term_set_vertical_mode(
+	ml_term_t *  term ,
+	ml_vertical_mode_t  mode
+	)
+{
+	term->vertical_mode = mode ;
+
+	return  1 ;
+}
+
+int
+ml_term_set_use_dynamic_comb(
+	ml_term_t *  term ,
+	int  flag
+	)
+{
+	term->use_dynamic_comb = flag ;
+
 	return  1 ;
 }
 
@@ -319,6 +343,10 @@ ml_term_get_pty_fd(
 	return  ml_pty_get_master_fd( term->pty) ;
 }
 
+/*
+ * Always return non-NULL value.
+ * XXX Static data can be returned. (Not reentrant)
+ */
 char *
 ml_term_get_slave_name(
 	ml_term_t *  term
@@ -326,7 +354,7 @@ ml_term_get_slave_name(
 {
 	if( term->pty == NULL)
 	{
-		return  NULL ;
+		return  "/dev/zombie" ;
 	}
 
 	return  ml_pty_get_slave_name( term->pty) ;
@@ -410,6 +438,15 @@ ml_term_unhighlight_cursor(
 	ml_line_t *  line ;
 	int  ret ;
 
+#ifdef  DEBUG
+	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG
+			" ml_term_unhighlight_cursor() should be called in visual context but"
+			" is called in logical context.\n") ;
+	}
+#endif
+
 	ml_screen_logical( term->screen) ;
 	
 	if( ( line = ml_screen_get_cursor_line( term->screen)) == NULL || ml_line_is_empty( line))
@@ -466,7 +503,10 @@ ml_term_set_modified_region_in_screen(
 	int  row ;
 	ml_line_t *  line ;
 
-	ml_screen_logical( term->screen) ;
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	{
+		ml_screen_logical( term->screen) ;
+	}
 
 	for( row = beg_row ; row < beg_row + nrows ; row ++)
 	{
@@ -476,8 +516,11 @@ ml_term_set_modified_region_in_screen(
 		}
 	}
 
-	/* ml_screen_render( term->screen) ; */
-	ml_screen_visual( term->screen) ;
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
+	{
+		/* ml_screen_render( term->screen) ; */
+		ml_screen_visual( term->screen) ;
+	}
 
 	return  1 ;
 }
@@ -493,7 +536,16 @@ ml_term_set_modified_lines(
 	int  row ;
 	ml_line_t *  line ;
 
-	if( term->vertical_mode)
+#ifdef  DEBUG
+	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG
+			" ml_term_set_modified_lines() should be called in visual context but"
+			" is called in logical context.\n") ;
+	}
+#endif
+
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		ml_screen_logical( term->screen) ;
 	}
@@ -506,7 +558,7 @@ ml_term_set_modified_lines(
 		}
 	}
 
-	if( term->vertical_mode)
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -525,7 +577,16 @@ ml_term_set_modified_lines_in_screen(
 	int  row ;
 	ml_line_t *  line ;
 
-	if( term->vertical_mode)
+#ifdef  DEBUG
+	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG
+			" ml_term_set_modified_lines_in_screen() should be called in visual "
+			" context but is called in logical context.\n") ;
+	}
+#endif
+
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		ml_screen_logical( term->screen) ;
 	}
@@ -538,7 +599,7 @@ ml_term_set_modified_lines_in_screen(
 		}
 	}
 
-	if( term->vertical_mode)
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -552,14 +613,23 @@ ml_term_set_modified_all_lines_in_screen(
 	ml_term_t *  term
 	)
 {
-	if( term->vertical_mode)
+#ifdef  DEBUG
+	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG
+			" ml_term_set_modified_all_lines_in_screen() should be called in "
+			" visual context but is called in logical context.\n") ;
+	}
+#endif
+
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		ml_screen_logical( term->screen) ;
 	}
 
 	ml_screen_set_modified_all( term->screen) ;
 
-	if( term->vertical_mode)
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -576,7 +646,16 @@ ml_term_updated_all(
 	int  row ;
 	ml_line_t *  line ;
 
-	if( term->vertical_mode)
+#ifdef  DEBUG
+	if( term->screen->logvis && ! term->screen->logvis->is_visual)
+	{
+		kik_debug_printf( KIK_DEBUG_TAG
+			" ml_term_updated_all() should be called in visual context but"
+			" is called in logical context.\n") ;
+	}
+#endif
+
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		ml_screen_logical( term->screen) ;
 	}
@@ -585,11 +664,11 @@ ml_term_updated_all(
 	{
 		if( ( line = ml_screen_get_line_in_screen( term->screen , row)))
 		{
-			ml_line_updated( line) ;
+			ml_line_set_updated( line) ;
 		}
 	}
 
-	if( term->vertical_mode)
+	if( ! ml_screen_logical_visual_is_reversible( term->screen))
 	{
 		/* ml_screen_render( term->screen) ; */
 		ml_screen_visual( term->screen) ;
@@ -611,6 +690,7 @@ ml_term_update_special_visual(
 	ml_logical_visual_t *  logvis ;
 	int  had_logvis = 0 ;
 	int  has_logvis = 0 ;
+	int  need_comb = 0 ;
 
 	if( term->shape)
 	{
@@ -621,19 +701,101 @@ ml_term_update_special_visual(
 	term->screen->use_dynamic_comb = 0 ;
 	had_logvis = ml_screen_delete_logical_visual( term->screen) ;
 
-	if( IS_ISCII_ENCODING( ml_term_get_encoding( term))
-	    || (ml_term_get_encoding( term) == ML_UTF8 && ! term->use_bidi && term->use_ind) )
+	if( term->use_dynamic_comb)
 	{
-		/*
-		 * It is impossible to process ISCII with other special visuals.
-		 */
+		if( ( logvis = ml_logvis_comb_new()))
+		{
+			if( ml_screen_add_logical_visual( term->screen , logvis))
+			{
+				has_logvis = 1 ;
+				need_comb = 1 ;
+				term->screen->use_dynamic_comb = 1 ;
+			}
+			else
+			{
+			#ifdef  DEBUG
+				kik_warn_printf( KIK_DEBUG_TAG
+					" ml_screen_add_logical_visual failed.\n") ;
+			#endif
 
+				(*logvis->delete)( logvis) ;
+			}
+		}
+	#ifdef  DEBUG
+		else
+		{
+			kik_warn_printf( KIK_DEBUG_TAG " ml_logvis_comb_new() failed.\n") ;
+		}
+	#endif
+	}
+
+	/* Vertical mode, BiDi and ISCII can't coexist. */
+
+	/* Similar if-else conditions exist in update_special_visual in x_screen.c. */
+	if( term->vertical_mode)
+	{
+		if( ( logvis = ml_logvis_vert_new( term->vertical_mode)))
+		{
+			if( ml_screen_add_logical_visual( term->screen , logvis))
+			{
+				has_logvis = 1 ;
+			}
+			else
+			{
+			#ifdef  DEBUG
+				kik_warn_printf( KIK_DEBUG_TAG
+					" ml_screen_add_logical_visual failed.\n") ;
+			#endif
+
+				(*logvis->delete)( logvis) ;
+			}
+		}
+	#ifdef  DEBUG
+		else
+		{
+			kik_warn_printf( KIK_DEBUG_TAG " ml_logvis_vert_new() failed.\n") ;
+		}
+	#endif
+	}
+	else if( term->use_bidi && ml_term_get_encoding( term) == ML_UTF8)
+	{
+		if( ( term->shape = ml_arabic_shape_new()) &&
+		    ( logvis = ml_logvis_bidi_new( term->bidi_mode)))
+		{
+			if( ml_screen_add_logical_visual( term->screen , logvis))
+			{
+				has_logvis = 1 ;
+				need_comb = 1 ;
+			}
+			else
+			{
+			#ifdef  DEBUG
+				kik_warn_printf( KIK_DEBUG_TAG
+					" ml_screen_add_logical_visual failed.\n") ;
+			#endif
+
+				(*logvis->delete)( logvis) ;
+			}
+		}
+	#ifdef  DEBUG
+		else
+		{
+			kik_warn_printf( KIK_DEBUG_TAG
+				" x_arabic_shape_new()/ml_logvis_bidi_new() failed.\n") ;
+		}
+	#endif
+	}
+	else if( IS_ISCII_ENCODING( ml_term_get_encoding( term))
+	         || (/* ! term->use_bidi && */ term->use_ind &&
+	             ml_term_get_encoding( term) == ML_UTF8) )
+	{
 		if( ( term->shape = ml_iscii_shape_new()) &&
 		    ( logvis = ml_logvis_iscii_new()))
 		{
 			if( ml_screen_add_logical_visual( term->screen , logvis))
 			{
 				has_logvis = 1 ;
+				need_comb = 1 ;
 			}
 			else
 			{
@@ -653,91 +815,11 @@ ml_term_update_special_visual(
 		}
 	#endif
 	}
-	else
+
+	if( need_comb && ! ml_term_is_using_char_combining( term))
 	{
-		if( term->use_dynamic_comb)
-		{
-			if( ( logvis = ml_logvis_comb_new()))
-			{
-				if( ml_screen_add_logical_visual( term->screen , logvis))
-				{
-					has_logvis = 1 ;
-					term->screen->use_dynamic_comb = 1 ;
-				}
-				else
-				{
-				#ifdef  DEBUG
-					kik_warn_printf( KIK_DEBUG_TAG
-						" ml_screen_add_logical_visual failed.\n") ;
-				#endif
-
-					(*logvis->delete)( logvis) ;
-				}
-			}
-		#ifdef  DEBUG
-			else
-			{
-				kik_warn_printf( KIK_DEBUG_TAG " ml_logvis_comb_new() failed.\n") ;
-			}
-		#endif
-		}
-		
-		if( term->vertical_mode)
-		{
-			if( ( logvis = ml_logvis_vert_new( term->vertical_mode)))
-			{
-				if( ml_screen_add_logical_visual( term->screen , logvis))
-				{
-					has_logvis = 1 ;
-				}
-				else
-				{
-				#ifdef  DEBUG
-					kik_warn_printf( KIK_DEBUG_TAG
-						" ml_screen_add_logical_visual failed.\n") ;
-				#endif
-
-					(*logvis->delete)( logvis) ;
-				}
-			}
-		#ifdef  DEBUG
-			else
-			{
-				kik_warn_printf( KIK_DEBUG_TAG " ml_logvis_vert_new() failed.\n") ;
-			}
-		#endif
-		}
-		else if( term->use_bidi && ml_term_get_encoding( term) == ML_UTF8)
-		{
-			if( ( term->shape = ml_arabic_shape_new()) &&
-		#if  1
-			    ( logvis = ml_logvis_bidi_new( 0 , term->bidi_mode)))
-		#else
-			    ( logvis = ml_logvis_bidi_new( 1 , term->bidi_mode)))
-		#endif
-			{
-				if( ml_screen_add_logical_visual( term->screen , logvis))
-				{
-					has_logvis = 1 ;
-				}
-				else
-				{
-				#ifdef  DEBUG
-					kik_warn_printf( KIK_DEBUG_TAG
-						" ml_screen_add_logical_visual failed.\n") ;
-				#endif
-
-					(*logvis->delete)( logvis) ;
-				}
-			}
-		#ifdef  DEBUG
-			else
-			{
-				kik_warn_printf( KIK_DEBUG_TAG
-					" x_arabic_shape_new()/ml_logvis_bidi_new() failed.\n") ;
-			}
-		#endif
-		}
+		kik_msg_printf( "Set use_combining=true forcibly.\n") ;
+		ml_term_set_use_char_combining( term , 1) ;
 	}
 
 	if( ! has_logvis)
@@ -781,101 +863,6 @@ ml_term_enter_backscroll_mode(
 	return  ml_enter_backscroll_mode( term->screen) ;
 }
 
-int
-ml_term_set_char_combining_flag(
-	ml_term_t *  term ,
-	int  flag
-	)
-{
-	term->parser->use_char_combining = flag ;
-
-	return  1 ;
-}
-
-int
-ml_term_is_using_char_combining(
-	ml_term_t *  term
-	)
-{
-	return  term->parser->use_char_combining ;
-}
-
-int
-ml_term_set_multi_col_char_flag(
-	ml_term_t *  term ,
-	int  flag
-	)
-{
-	term->parser->use_multi_col_char = flag ;
-
-	return  1 ;
-}
-
-int
-ml_term_is_using_multi_col_char(
-	ml_term_t *  term
-	)
-{
-	return  term->parser->use_multi_col_char ;
-}
-
-int
-ml_term_set_mouse_report(
-	ml_term_t *  term ,
-	int  flag
-	)
-{
-	term->is_mouse_pos_sending = flag ;
-
-	return  1 ;
-}
-
-int
-ml_term_is_mouse_pos_sending(
-	ml_term_t *  term
-	)
-{
-	return  term->is_mouse_pos_sending ;
-}
-
-int
-ml_term_set_app_keypad(
-	ml_term_t *  term ,
-	int  flag
-	)
-{
-	term->is_app_keypad = flag ;
-
-	return  1 ;
-}
-
-int
-ml_term_is_app_keypad(
-	ml_term_t *  term
-	)
-{
-	return  term->is_app_keypad ;
-}
-
-int
-ml_term_set_app_cursor_keys(
-	ml_term_t *  term ,
-	int  flag
-	)
-{
-	term->is_app_cursor_keys = flag ;
-
-	return  1 ;
-}
-
-int
-ml_term_is_app_cursor_keys(
-	ml_term_t *  term
-	)
-{
-	return  term->is_app_cursor_keys ;
-}
-
 int
 ml_term_set_window_name(
 	ml_term_t *  term ,
@@ -912,30 +899,6 @@ ml_term_set_icon_path(
 	return 1 ;
 }
 
-char *
-ml_term_window_name(
-	ml_term_t *  term
-	)
-{
-	return  term->win_name ;
-}
-
-char *
-ml_term_icon_name(
-	ml_term_t *  term
-	)
-{
-	return  term->icon_name ;
-}
-
-char *
-ml_term_icon_path(
-	ml_term_t *  term
-	)
-{
-	return term->icon_path ;
-}
-
 int
 ml_term_start_config_menu(
 	ml_term_t *  term ,
@@ -946,5 +909,5 @@ ml_term_start_config_menu(
 	)
 {
 	return  ml_config_menu_start( &term->config_menu , cmd_path ,
-			x , y , display, ml_pty_get_slave_fd( term->pty)) ;
+				x , y , display , term->pty) ;
 }
