@@ -34,24 +34,22 @@ typedef struct ml_term
 	 * public(read/write)
 	 */
 	ml_shape_t *  shape ;
-	ml_vertical_mode_t  vertical_mode ;
-	ml_bidi_mode_t  bidi_mode ;
-	
-	int8_t  use_bidi ;
-	int8_t  use_ind ;
-	int8_t  use_dynamic_comb ;
 
 	/*
 	 * private
 	 */
+	ml_vertical_mode_t  vertical_mode ;
+	ml_bidi_mode_t  bidi_mode ;
+
 	char *  win_name ;
 	char *  icon_name ;
 	char *  icon_path ;
 
+	int8_t  use_bidi ;
+	int8_t  use_ind ;
+	int8_t  use_dynamic_comb ;
+
 	int8_t  is_auto_encoding ;
-	int8_t  is_mouse_pos_sending ;
-	int8_t  is_app_keypad ;
-	int8_t  is_app_cursor_keys ;
 
 	int8_t  is_attached ;
 
@@ -78,7 +76,7 @@ int  ml_term_attach( ml_term_t *  term , ml_xterm_event_listener_t *  xterm_list
 
 int  ml_term_detach( ml_term_t *  term) ;
 
-int  ml_term_is_attached( ml_term_t *  term) ;
+#define  ml_term_is_attached( term)  ((term)->is_attached)
 
 #define  ml_term_parse_vt100_sequence( term)  ml_parse_vt100_sequence( (term)->parser)
 
@@ -89,11 +87,31 @@ int  ml_term_is_attached( ml_term_t *  term) ;
 
 int  ml_term_set_auto_encoding( ml_term_t *  term , int  is_auto_encoding) ;
 
-int  ml_term_is_auto_encoding( ml_term_t *  term) ;
+#define  ml_term_is_auto_encoding( term)  ((term)->is_auto_encoding)
 
 #define  ml_term_set_unicode_policy( term , policy) \
 		ml_vt100_parser_set_unicode_policy( (term)->parser , policy)
 
+int  ml_term_set_use_bidi( ml_term_t *  term , int  flag) ;
+
+#define  ml_term_is_using_bidi( term)  ((term)->use_bidi)
+
+int  ml_term_set_bidi_mode( ml_term_t *  term , ml_bidi_mode_t  mode) ;
+
+#define  ml_term_get_bidi_mode( term)  ((term)->bidi_mode)
+
+int  ml_term_set_use_ind( ml_term_t *  term , int  flag) ;
+
+#define  ml_term_is_using_ind( term)  ((term)->use_ind)
+
+int  ml_term_set_vertical_mode( ml_term_t *  term , ml_vertical_mode_t  mode) ;
+
+#define  ml_term_get_vertical_mode( term)  ((term)->vertical_mode)
+
+int  ml_term_set_use_dynamic_comb( ml_term_t *  term , int  flag) ;
+
+#define  ml_term_is_using_dynamic_comb( term)  ((term)->use_dynamic_comb)
+
 #define  ml_term_convert_to( term , dst , len , _parser) \
 		ml_vt100_parser_convert_to( (term)->parser , dst , len , _parser)
 
@@ -101,8 +119,6 @@ int  ml_term_is_auto_encoding( ml_term_t *  term) ;
 
 #define  ml_term_init_encoding_conv( term)  ml_init_encoding_conv( (term)->parser)
 
-int  ml_term_set_logging_vt_seq( ml_term_t *  term , int  flag) ;
-
 int  ml_term_get_pty_fd( ml_term_t *  term) ;
 
 char *  ml_term_get_slave_name( ml_term_t *  term) ;
@@ -113,6 +129,10 @@ size_t  ml_term_write( ml_term_t *  term , u_char *  buf , size_t  len , int  to
 
 int  ml_term_flush( ml_term_t *  term) ;
 
+/* Must be called in visual context. */
+#define  ml_term_write_loopback( term , buf , len) \
+		ml_parse_vt100_write_loopback( (term)->parser , buf , len)
+
 int  ml_term_resize( ml_term_t *  term , u_int  cols , u_int  rows) ;
 
 #define  ml_term_cursor_col( term)  ml_screen_cursor_col( (term)->screen)
@@ -170,6 +190,9 @@ int  ml_term_updated_all( ml_term_t *  term) ;
 
 int  ml_term_update_special_visual( ml_term_t *  term) ;
 
+#define  ml_term_logical_visual_is_reversible( term) \
+		ml_screen_logical_visual_is_reversible( (term)->screen)
+
 #define  ml_term_is_backscrolling( term)  ml_screen_is_backscrolling( (term)->screen)
 
 #define  ml_term_set_backscroll_mode( term , mode)  ml_set_backscroll_mode( (term)->screen , mode)
@@ -216,31 +239,39 @@ int  ml_term_enter_backscroll_mode( ml_term_t *  term) ;
 		ml_screen_get_word_region( (term)->screen , beg_char_index , beg_row , \
 			end_char_index , end_row , base_char_index , base_row)
 
-int  ml_term_set_char_combining_flag( ml_term_t *  term , int  flag) ;
-
-int  ml_term_is_using_char_combining( ml_term_t *  term) ;
-
-int  ml_term_set_multi_col_char_flag( ml_term_t *  term , int  flag) ;
-
-int  ml_term_is_using_multi_col_char( ml_term_t *  term) ;
-
 #define  ml_term_set_col_size_of_width_a( term , col_size_a) \
 		ml_vt100_parser_set_col_size_of_width_a( (term)->parser , col_size_a)
 
 #define  ml_term_get_col_size_of_width_a( term) \
 		ml_vt100_parser_get_col_size_of_width_a( (term)->parser)
 
-int  ml_term_set_mouse_report( ml_term_t *  term , int  flag) ;
+#define  ml_term_set_use_char_combining( term , flag) \
+		ml_vt100_parser_set_use_char_combining( (term)->parser , flag)
 
-int  ml_term_is_mouse_pos_sending( ml_term_t *  term) ;
+#define  ml_term_is_using_char_combining( term) \
+		ml_vt100_parser_is_using_char_combining((term)->parser)
 
-int  ml_term_set_app_keypad( ml_term_t *  term , int  flag) ;
+#define  ml_term_set_use_multi_col_char( term , flag) \
+		ml_vt100_parser_set_use_multi_col_char( (term)->parser , flag)
 
-int  ml_term_is_app_keypad( ml_term_t *  term) ;
+#define  ml_term_is_using_multi_col_char( term) \
+		ml_vt100_parser_is_using_multi_col_char((term)->parser)
 
-int  ml_term_set_app_cursor_keys( ml_term_t *  term , int  flag) ;
+#define  ml_term_is_logging_vt_seq( term)  ml_vt100_parser_is_logging_vt_seq( (term)->parser)
 
-int  ml_term_is_app_cursor_keys( ml_term_t *  term) ;
+#define  ml_term_set_logging_vt_seq( term , flag) \
+		ml_vt100_parser_set_logging_vt_seq( (term)->parser , flag)
+
+#define  ml_term_get_mouse_report_mode( term) \
+		ml_vt100_parser_get_mouse_report_mode((term)->parser)
+
+#define  ml_term_is_app_keypad( term) \
+		ml_vt100_parser_is_app_keypad((term)->parser)
+
+#define  ml_term_is_app_cursor_keys( term) \
+		ml_vt100_parser_is_app_cursor_keys((term)->parser)
+
+#define  ml_term_is_app_escape( term)  ml_vt100_parser_is_app_escape((term)->parser)
 
 int  ml_term_set_window_name( ml_term_t *  term , char *  name) ;
 
@@ -248,11 +279,14 @@ int  ml_term_set_icon_name( ml_term_t *  term , char *  name) ;
 
 int  ml_term_set_icon_path( ml_term_t *  term , char *  path) ;
 
-char *  ml_term_window_name( ml_term_t *  term) ;
+#define  ml_term_window_name( term)  ((term)->win_name)
 
-char *  ml_term_icon_name( ml_term_t *  term) ;
+#define  ml_term_icon_name( term)  ((term)->icon_name)
 
-char *  ml_term_icon_path( ml_term_t *  term) ;
+#define  ml_term_icon_path( term)  ((term)->icon_path)
+
+#define  ml_term_is_bracketed_paste_mode( term) \
+		ml_vt100_parser_is_bracketed_paste_mode((term)->parser)
 
 int  ml_term_start_config_menu( ml_term_t *  term , char *  cmd_path ,
 		int  x , int  y , char *  display) ;
