@@ -72,7 +72,6 @@ bidi_flag_checked(
 	return  1 ;
 }
 
-
 /*
  *  ********  procedures when buttons are clicked  ********
  */
@@ -181,12 +180,174 @@ full_reset_clicked(
 	gpointer  data
 	)
 {
-	mc_set_str_value( "full_reset" , "") ;
-	mc_flush(mc_io_set);
+	mc_exec( "full_reset") ;
 
 	return  1 ;
 }
 
+#ifdef  USE_LIBSSH2
+
+#define  MY_RESPONSE_RETURN  1
+#define  MY_RESPONSE_EXIT    2
+#define  MY_RESPONSE_SEND    3
+#define  MY_RESPONSE_RECV    4
+
+static void
+drag_data_received(
+	GtkWidget *  widget ,
+	GdkDragContext *  context ,
+	gint  x ,
+	gint  y ,
+	GtkSelectionData *  data ,
+	guint  info ,
+	guint  time
+	)
+{
+	gchar **  uris ;
+	gchar *  filename ;
+
+	uris = g_uri_list_extract_uris( data->data) ;
+	filename = g_filename_from_uri( uris[0] , NULL , NULL) ;
+	gtk_entry_set_text( GTK_ENTRY(widget) , filename) ;
+	g_free( filename) ;
+	g_strfreev( uris) ;
+}
+
+static gint
+ssh_scp_clicked(
+	GtkWidget *  widget ,
+	gpointer  data
+	)
+{
+	GtkWidget *  dialog ;
+	GtkWidget *  content_area ;
+	GtkWidget *  hbox ;
+	GtkWidget *  label ;
+	GtkWidget *  local_entry ;
+	GtkWidget *  remote_entry ;
+	gint  res ;
+	GtkTargetEntry  local_targets[] =
+	{
+		{ "text/uri-list" , 0 , 0 } ,
+	} ;
+
+	gtk_widget_hide_all( gtk_widget_get_toplevel( widget)) ;
+	
+	dialog = gtk_dialog_new() ;
+	gtk_window_set_title( GTK_WINDOW(dialog) , "mlconfig") ;
+	gtk_dialog_add_button( GTK_DIALOG(dialog) , _("Send") , MY_RESPONSE_SEND) ;
+	gtk_dialog_add_button( GTK_DIALOG(dialog) , _("Recv") , MY_RESPONSE_RECV) ;
+	gtk_dialog_add_button( GTK_DIALOG(dialog) , _("Return") , MY_RESPONSE_RETURN) ;
+	gtk_dialog_add_button( GTK_DIALOG(dialog) , _("Exit") , MY_RESPONSE_EXIT) ;
+
+	content_area = GTK_DIALOG(dialog)->vbox ;
+	
+	hbox = gtk_hbox_new(FALSE , 0) ;
+	gtk_widget_show(hbox) ;
+
+	label = gtk_label_new( _("Local")) ;
+	gtk_widget_show(label) ;
+	gtk_widget_set_usize( label , 70 , 0) ;
+	gtk_box_pack_start(GTK_BOX(hbox) , label , FALSE , TRUE , 1) ;
+	
+	local_entry = gtk_entry_new() ;
+	gtk_widget_show(local_entry) ;
+	gtk_widget_set_usize( local_entry , 280 , 0) ;
+	gtk_drag_dest_set( local_entry , GTK_DEST_DEFAULT_ALL ,
+		local_targets , 1 , GDK_ACTION_COPY) ;
+	g_signal_connect( local_entry , "drag-data-received" ,
+			G_CALLBACK(drag_data_received) , NULL) ;
+	gtk_box_pack_start(GTK_BOX(hbox) , local_entry , FALSE , TRUE , 1) ;
+
+	gtk_container_add( GTK_CONTAINER(content_area) , hbox) ;
+
+	hbox = gtk_hbox_new(FALSE , 0) ;
+	gtk_widget_show(hbox) ;
+
+	label = gtk_label_new( _("Remote")) ;
+	gtk_widget_show(label) ;
+	gtk_widget_set_usize( label , 70 , 0) ;
+	gtk_box_pack_start(GTK_BOX(hbox) , label , FALSE , TRUE , 1) ;
+
+	remote_entry = gtk_entry_new() ;
+	gtk_widget_show(remote_entry) ;
+	gtk_widget_set_usize( remote_entry , 280 , 0) ;
+	gtk_box_pack_start(GTK_BOX(hbox) , remote_entry , FALSE , TRUE , 1) ;
+
+	gtk_container_add( GTK_CONTAINER(content_area) , hbox) ;
+
+	while( (res = gtk_dialog_run( GTK_DIALOG(dialog))) >= MY_RESPONSE_SEND)
+	{
+		char *  cmd ;
+		const gchar *  local_path ;
+		const gchar *  remote_path ;
+
+		local_path = gtk_entry_get_text( GTK_ENTRY(local_entry)) ;
+		remote_path = gtk_entry_get_text( GTK_ENTRY(remote_entry)) ;
+
+		if( ( cmd = alloca( 28 + strlen( local_path) + strlen( remote_path))))
+		{
+			char *  p ;
+
+			if( res == MY_RESPONSE_SEND)
+			{
+				if( ! *local_path)
+				{
+					kik_msg_printf( "Local file path to send "
+						"is not specified.\n") ;
+
+					continue ;
+				}
+
+				sprintf( cmd , "scp \"local:%s\" \"remote:%s\" UTF8" ,
+						local_path , remote_path) ;
+			}
+			else /* if( res == MY_RESPONSE_RECV) */
+			{
+				if( ! *remote_path)
+				{
+					kik_msg_printf( "Remote file path to receive "
+						"is not specified.\n") ;
+
+					continue ;
+				}
+
+				sprintf( cmd , "scp \"remote:%s\" \"local:%s\" UTF8" ,
+						remote_path , local_path) ;
+			}
+
+			p = cmd + strlen(cmd) - 2 ;
+			if( *p == '\\')
+			{
+				/*
+				 * Avoid to be parsed as follows.
+				 * "local:c:\foo\bar\" => local:c:\foo\bar"
+				 */
+				*(p++) = '\"' ;
+				*p = '\0' ;
+			}
+			
+			mc_exec( cmd) ;
+		}
+	}
+
+	if( res == MY_RESPONSE_EXIT)
+	{
+		gtk_main_quit() ;
+
+		return  FALSE ;
+	}
+	else /* if( res == MY_RESPONSE_RETURN) */
+	{
+		gtk_widget_destroy( dialog) ;
+		gtk_widget_show_all( gtk_widget_get_toplevel( widget)) ;
+
+		return  TRUE ;
+	}
+}
+
+#endif	/* USE_LIBSSH2 */
+
 static gint
 pty_new_button_clicked(
 	GtkWidget *  widget ,
@@ -295,6 +456,27 @@ full_reset(void)
 	return frame;
 }
 
+#ifdef  USE_LIBSSH2
+static GtkWidget *
+ssh_scp(void)
+{
+	GtkWidget *  frame ;
+	GtkWidget *  hbox ;
+
+	frame = gtk_frame_new( _("SSH SCP")) ;
+	gtk_widget_show(frame) ;
+
+	hbox = gtk_hbox_new( FALSE , 5) ;
+	gtk_container_set_border_width(GTK_CONTAINER(hbox) , 5) ;
+	gtk_widget_show(hbox) ;
+	gtk_container_add(GTK_CONTAINER(frame) , hbox) ;
+
+	addbutton(_("SSH SCP"), ssh_scp_clicked, hbox);
+
+	return frame;
+}
+#endif	/* USE_LIBSSH2 */
+
 static GtkWidget *
 pty_list(void)
 {
@@ -375,7 +557,11 @@ show(void)
 	gtk_box_pack_start( GTK_BOX(hbox) , frame , TRUE , TRUE , 5) ;
 	frame = full_reset();
 	gtk_box_pack_start( GTK_BOX(hbox) , frame , TRUE , TRUE , 5) ;
-	
+#ifdef  USE_LIBSSH2
+	frame = ssh_scp() ;
+	gtk_box_pack_start( GTK_BOX(hbox) , frame , TRUE , TRUE , 5) ;
+#endif
+
 	frame = pty_list() ;
 	gtk_box_pack_start( GTK_BOX(vbox) , frame , FALSE , FALSE , 0) ;
 	
@@ -405,9 +591,6 @@ show(void)
 
 	if (!(bidi_flag = mc_flag_config_widget_new(MC_FLAG_BIDI)))
 	    return 0;
-#ifndef USE_FRIBIDI
-	gtk_widget_set_sensitive(bidi_flag, 0);
-#endif
 	gtk_widget_show(bidi_flag);
 	gtk_box_pack_start(GTK_BOX(hbox), bidi_flag, TRUE, TRUE, 0);
 
@@ -416,15 +599,9 @@ show(void)
 	gtk_widget_show(config_widget);
 	gtk_box_pack_start(GTK_BOX(hbox), config_widget, TRUE, TRUE, 0);
 
-#ifdef  USE_IND
-#ifdef  USE_FRIBIDI
 	gtk_signal_connect(GTK_OBJECT(bidi_flag), "toggled" ,
 		GTK_SIGNAL_FUNC(bidi_flag_checked), config_widget);
 	gtk_widget_set_sensitive(config_widget, ! GTK_TOGGLE_BUTTON(bidi_flag)->active);
-#endif
-#else
-	gtk_widget_set_sensitive(config_widget, 0);
-#endif
 
 	if(!(config_widget = mc_flag_config_widget_new(MC_FLAG_COMB)))
 	    return 0;
