@@ -54,14 +54,6 @@ static cs_info_t  cs_info_table[] =
 	{ "ISO10646_1" , { "iso10646-1" , NULL , NULL , } , } ,
 	{ "ISO10646_1_BIWIDTH" , { "iso10646-1" , NULL , NULL , } , } ,
 
-#if  0
-	/*
-	 * Font of these charsets is not manipulated by mlconfig at present.
-	 */
-	{ "ISO10646_UCS2_1" , { "iso10646-1" , NULL , NULL , } , } ,
-	{ "ISO10646_UCS2_1_BIWIDTH" , { "iso10646-1"  , NULL , NULL , } , } ,
-#endif
-
 	{ "DEC_SPECIAL" , { "iso8859-1" , NULL , NULL , } , } ,
 	{ "ISO8859_1" , { "iso8859-1" , NULL , NULL , } , } ,
 	{ "ISO8859_2" , { "iso8859-2" , NULL , NULL , } , } ,
@@ -286,12 +278,7 @@ aa_flag_checked(
 		if( ! GTK_TOGGLE_BUTTON(xft_flag)->active &&
 		    ! GTK_TOGGLE_BUTTON(cairo_flag)->active)
 		{
-		#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
-		#ifdef  USE_TYPE_XFT
 			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(xft_flag) , 1) ;
-		#else
-			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cairo_flag) , 1) ;
-		#endif
 		
 			reset_fontname_list() ;
 			gtk_entry_set_text(GTK_ENTRY(fontname_entry) ,
@@ -299,7 +286,6 @@ aa_flag_checked(
 					mc_get_font_name( get_font_file() , new_fontsize ,
 						get_correct_cs( selected_cs)) ,
 					-1 , NULL , NULL , NULL) ) ;
-		#endif
 		}
 	}
 	
@@ -741,7 +727,6 @@ mc_font_config_widget_new(void)
 	GtkWidget *  vbox ;
 	GtkWidget *  hbox ;
 	GtkWidget *  combo ;
-	GtkWidget *  button ;
 	char *  fontlist[] =
 	{
 		"6" , "7" , "8" , "9" , "10" ,
@@ -790,7 +775,7 @@ mc_font_config_widget_new(void)
 	}
 
 	aa_flag = mc_flag_config_widget_new( MC_FLAG_AA) ;
-#if  ! defined(USE_TYPE_XFT) && ! defined(USE_TYPE_CAIRO) && ! defined(USE_WIN32GUI)
+#if  ! defined(USE_WIN32GUI)
 	gtk_widget_set_sensitive(aa_flag, 0) ;
 #endif
 	gtk_widget_show( aa_flag) ;
@@ -798,15 +783,6 @@ mc_font_config_widget_new(void)
 	gtk_signal_connect(GTK_OBJECT(aa_flag) , "toggled" ,
 		GTK_SIGNAL_FUNC(aa_flag_checked) , NULL) ;
 	
-	if( GTK_TOGGLE_BUTTON(aa_flag)->active)
-	{
-	#if  defined(USE_TYPE_XFT)
-		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(xft_flag) , 1) ;
-	#elif  defined(USE_TYPE_CAIRO)
-		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(cairo_flag) , 1) ;
-	#endif
-	}
-	
 	vcol_flag = mc_flag_config_widget_new( MC_FLAG_VCOL) ;
 	gtk_widget_show( vcol_flag) ;
 	gtk_box_pack_start(GTK_BOX(hbox) , vcol_flag , TRUE , TRUE , 0) ;
@@ -816,9 +792,9 @@ mc_font_config_widget_new(void)
 	gtk_box_pack_start(GTK_BOX(vbox) , hbox , TRUE , TRUE , 0) ;
 
 
-	button = mc_vertical_config_widget_new() ;
-	gtk_widget_show(button) ;
-	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
+	hbox = mc_vertical_config_widget_new() ;
+	gtk_widget_show(hbox) ;
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 
 
 	hbox = gtk_hbox_new(FALSE , 0) ;
