@@ -11,39 +11,11 @@
 
 /* --- static functions --- */
 
-/*
- * Not used for now.
- */
-#if  0
-static int
-char_combining_is_supported(
-	mkf_charset_t  cs
-	)
-{
-	/*
-	 * some thai fonts (e.g. -thai-fixed-medium-r-normal--14-100-100-100-m-70-tis620.2529-1
-	 * distributed by ZzzThai(http://zzzthai.fedu.uec.ac.jp/ZzzThai/))
-	 * automatically draws combining chars ...
-	 */
-	if( cs == TIS620_2533)
-	{
-		return  1 ;
-	}
-	else
-	{
-		return  0 ;
-	}
-}
-#else
-#define  char_combining_is_supported(cs)  0
-#endif
-
-
 /*
  * drawing string
  */
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 
 static int
 fc_draw_combining_chars(
@@ -83,7 +55,7 @@ fc_draw_combining_chars(
 		}
 		else
 		{
-			FcChar32  xch ;
+			/* FcChar32 */ u_int32_t  xch ;
 
 			char  ucs4_bytes[4] ;
 
@@ -127,8 +99,8 @@ fc_draw_str(
 	int  start_draw ;
 	int  end_of_str ;
 	u_int	current_width ;
-	FcChar8 *  str8 ;
-	FcChar32 *  str32 ;
+	/* FcChar8 */ u_int8_t *  str8 ;
+	/* FcChar32 */ u_int32_t *  str32 ;
 	u_int	str_len ;
 	int  state ;
 	int  next_state ;
@@ -208,12 +180,12 @@ fc_draw_str(
 
 	is_underlined = ml_char_is_underlined( &chars[count]) ;
 
-	if( ( str8 = alloca( sizeof( FcChar8) * num_of_chars)) == NULL)
+	if( ( str8 = alloca( sizeof( /* FcChar8 */ u_int8_t) * num_of_chars)) == NULL)
 	{
 		return	0 ;
 	}
 
-	if( ( str32 = alloca( sizeof( FcChar32) * num_of_chars)) == NULL)
+	if( ( str32 = alloca( sizeof( /* FcChar32 */ u_int32_t) * num_of_chars)) == NULL)
 	{
 		return  0 ;
 	}
@@ -371,7 +343,20 @@ fc_draw_str(
 			{
 				fc_draw_combining_chars( window , font_man , color_man ,
 					comb_chars , comb_size ,
-					current_width - ch_width , y + height_to_baseline) ;
+				/*
+				 * 'current_width' is for some thai fonts which automatically
+				 * draw combining chars.
+				 * e.g.)
+				 *  -thai-fixed-medium-r-normal--14-100-100-100-m-70-tis620.2529-1
+				 *  (distributed by ZzzThai http://zzzthai.fedu.uec.ac.jp/ZzzThai/)
+				 *  win32 unicode font.
+				 */
+				#if  0
+					current_width
+				#else
+					x
+				#endif
+					, y + height_to_baseline) ;
 			}
 
 			if( is_underlined)
@@ -425,7 +410,7 @@ fc_draw_str(
 
 #endif
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 
 static u_int		/* Return written size of XChar2b */
 convert_ucs4_to_utf16(
@@ -877,12 +862,20 @@ xcore_draw_str(
 			{
 				xcore_draw_combining_chars( window , font_man , color_man ,
 					comb_chars , comb_size ,
+				/*
+				 * 'current_width' is for some thai fonts which automatically
+				 * draw combining chars.
+				 * e.g.)
+				 *  -thai-fixed-medium-r-normal--14-100-100-100-m-70-tis620.2529-1
+				 *  (distributed by ZzzThai http://zzzthai.fedu.uec.ac.jp/ZzzThai/)
+				 *  win32 unicode font.
+				 */
 				#if  0
-					current_width ,	/* Thai comb chars needs this in win32. */
+					current_width
 				#else
-					current_width - ch_width ,
+					x
 				#endif
-					y + height_to_baseline) ;
+					, y + height_to_baseline) ;
 			}
 
 			if( is_underlined)
@@ -960,7 +953,7 @@ x_draw_str(
 	default:
 		return  0 ;
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	case  TYPE_XFT:
 	case  TYPE_CAIRO:
 		if( ! fc_draw_str( window , font_man , color_man , &updated_width , chars , num_of_chars ,
@@ -972,7 +965,7 @@ x_draw_str(
 		break ;
 #endif
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	case  TYPE_XCORE:
 		if( ! xcore_draw_str( window , font_man , color_man , &updated_width , chars , num_of_chars ,
 			x , y , height , height_to_baseline , top_margin , bottom_margin))
@@ -1009,7 +1002,7 @@ x_draw_str_to_eol(
 	default:
 		return  0 ;
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	case  TYPE_XFT:
 	case  TYPE_CAIRO:
 		if( ! fc_draw_str( window , font_man , color_man , &updated_width , chars , num_of_chars ,
@@ -1021,7 +1014,7 @@ x_draw_str_to_eol(
 		break ;
 #endif
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	case  TYPE_XCORE:
 		if( ! xcore_draw_str( window , font_man , color_man , &updated_width , chars , num_of_chars ,
 			x , y , height , height_to_baseline , top_margin , bottom_margin))
