@@ -4,29 +4,22 @@
 
 #include  "x_font.h"
 
-#include  <string.h>		/* memset/strncasecmp */
-#ifdef  USE_TYPE_CAIRO
-#include  <cairo/cairo-xlib.h>
-#include  <cairo/cairo-ft.h>
-#endif
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_str.h>	/* kik_snprintf */
 #include  <kiklib/kik_mem.h>	/* alloca */
-#include  <kiklib/kik_str.h>	/* kik_str_sep/kik_str_to_int */
+#include  <kiklib/kik_str.h>	/* kik_str_sep/kik_str_to_int/memset/strncasecmp */
 #include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN/K_MIN */
 #include  <kiklib/kik_locale.h>	/* kik_get_lang() */
 #include  <mkf/mkf_ucs4_map.h>
 #include  <ml_char_encoding.h>	/* ml_is_msb_set */
-#include  <ml_char.h>		/* UTF_MAX_SIZE */
+#include  "x_type_loader.h"
+
 
 #define  FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p) \
 	for( (font_encoding_p) = &csinfo->encoding_names[0] ; *(font_encoding_p) ; (font_encoding_p) ++)
 #define  DIVIDE_ROUNDING(a,b)  ( ((int)((a)*10 + (b)*5)) / ((int)((b)*10)) )
 #define  DIVIDE_ROUNDINGUP(a,b) ( ((int)((a)*10 + (b)*10 - 1)) / ((int)((b)*10)) )
 
-/* Be careful not to round down 5.99999... to 5 */
-#define  DOUBLE_ROUNDUP_TO_INT(a)  ((int)((a) + 0.9))
-
 #if  0
 #define  __DEBUG
 #endif
@@ -56,7 +49,6 @@ typedef struct  cs_info
 static cs_info_t  cs_info_table[] =
 {
 	{ ISO10646_UCS4_1 , { "iso10646-1" , NULL , NULL , } , } ,
-	{ ISO10646_UCS2_1 , { "iso10646-1" , NULL , NULL , } , } ,
 
 	{ DEC_SPECIAL , { "iso8859-1" , NULL , NULL , } , } ,
 	{ ISO8859_1_R , { "iso8859-1" , NULL , NULL , } , } ,
@@ -152,10 +144,8 @@ static cs_info_t  cs_info_table[] =
 } ;
 
 static int  compose_dec_special_font ;
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
-static char *  fc_size_type = FC_PIXEL_SIZE ;
+static int  use_point_size_for_fc ;
 static double  dpi_for_fc ;
-#endif
 
 
 /* --- static functions --- */
@@ -183,6 +173,41 @@ get_cs_info(
 	return  NULL ;
 }
 
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE)
+static int
+xft_unset_font(
+	x_font_t *  font
+	)
+{
+	int (*func)( x_font_t *) ;
+
+	if( ! ( func = x_load_type_xft_func( X_UNSET_FONT)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font) ;
+}
+#endif
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE)
+static int
+cairo_unset_font(
+	x_font_t *  font
+	)
+{
+	int (*func)( x_font_t *) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_UNSET_FONT)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font) ;
+}
+#endif
+
+
 static int
 set_decsp_font(
 	x_font_t *  font
@@ -191,14 +216,21 @@ set_decsp_font(
 	/*
 	 * freeing font->xfont or font->xft_font
 	 */
-#ifdef  USE_TYPE_XFT
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	if( font->xft_font)
 	{
-		XftFontClose( font->display , font->xft_font) ;
-		font->xft_font = NULL ;
+		xft_unset_font( font) ;
 	}
 #endif
-#ifdef  USE_TYPE_XCORE
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	if( font->cairo_font)
+	{
+		cairo_unset_font( font) ;
+	}
+#endif
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	if( font->xfont)
 	{
 		XFreeFont( font->display , font->xfont) ;
@@ -221,841 +253,11 @@ set_decsp_font(
 	return  1 ;
 }
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 
-static int
-parse_fc_font_name(
-	char **  font_family ,
-	int *  font_weight ,	/* if weight is not specified in font_name , not changed. */
-	int *  font_slant ,	/* if slant is not specified in font_name , not changed. */
-	double *  font_size ,	/* if size is not specified in font_name , not changed. */
-	char **  font_encoding ,/* if encoding is not specified in font_name , not changed. */
-	u_int *  percent ,	/* if percent is not specified in font_name , not changed. */
-	char *  font_name	/* modified by this function. */
-	)
-{
-	char *  p ;
-	size_t  len ;
-	
-	/*
-	 * XftFont format.
-	 * [Family]( [WEIGHT] [SLANT] [SIZE]-[Encoding]:[Percentage])
-	 */
-
-	*font_family = font_name ;
-	
-	p = font_name ;
-	while( 1)
-	{
-		if( *p == '\\' && *(p + 1))
-		{
-			/*
-			 * It seems that XftFont format allows hyphens to be escaped.
-			 * (e.g. Foo\-Bold-iso10646-1)
-			 */
-
-			/* skip backslash */
-			p ++ ;
-		}
-		else if( *p == '\0')
-		{
-			/* encoding and percentage is not specified. */
-			
-			*font_name = '\0' ;
-			
-			break ;
-		}
-		else if( *p == '-')
-		{
-			/* Parsing "-[Encoding]:[Percentage]" */
-			
-			*font_name = '\0' ;
-
-			*font_encoding = ++p ;
-			
-			kik_str_sep( &p , ":") ;
-			if( p)
-			{
-				if( ! kik_str_to_uint( percent , p))
-				{
-				#ifdef  DEBUG
-					kik_warn_printf( KIK_DEBUG_TAG
-						" Percentage(%s) is illegal.\n" , p) ;
-				#endif
-				}
-			}
-			
-			break ;
-		}
-		else if( *p == ':')
-		{
-			/* Parsing ":[Percentage]" */
-			
-			*font_name = '\0' ;
-			
-			if( ! kik_str_to_uint( percent , p + 1))
-			{
-			#ifdef  DEBUG
-				kik_warn_printf( KIK_DEBUG_TAG
-					" Percentage(%s) is illegal.\n" , p + 1) ;
-			#endif
-			}
-			
-			break ;
-		}
-
-		*(font_name++) = *(p++) ;
-	}
-
-	/*
-	 * Parsing "[Family] [WEIGHT] [SLANT] [SIZE]".
-	 * Following is the same as x_font_win32.c:parse_font_name()
-	 * except FC_*.
-	 */
-
-#if  0
-	kik_debug_printf( "Parsing %s for [Family] [Weight] [Slant]\n" , *font_family) ;
-#endif
-
-	p = kik_str_chop_spaces( *font_family) ;
-	len = strlen( p) ;
-	while( len > 0)
-	{
-		size_t  step = 0 ;
-
-		if( *p == ' ')
-		{
-			char *  orig_p ;
-
-			orig_p = p ;
-			do
-			{
-				p ++ ;
-				len -- ;
-			}
-			while( *p == ' ') ;
-
-			if( len == 0)
-			{
-				*orig_p = '\0' ;
-				
-				break ;
-			}
-			else
-			{
-				int  count ;
-				struct
-				{
-					char *  style ;
-					int  weight ;
-					int  slant ;
-
-				} styles[] =
-				{
-					/*
-					 * Portable styles.
-					 */
-					/* slant */
-					{ "italic" , 0 , FC_SLANT_ITALIC , } ,
-					/* weight */
-					{ "bold" , FC_WEIGHT_BOLD , 0 , } ,
-
-					/*
-					 * Hack for styles which can be returned by
-					 * gtk_font_selection_dialog_get_font_name().
-					 */
-					/* slant */
-					{ "oblique" , 0 , FC_SLANT_OBLIQUE , } ,
-					/* weight */
-					{ "light" , /* e.g. "Bookman Old Style Light" */
-						FC_WEIGHT_LIGHT , 0 , } ,
-					{ "semi-bold" , FC_WEIGHT_DEMIBOLD , 0 , } ,
-					{ "heavy" , /* e.g. "Arial Black Heavy" */
-						FC_WEIGHT_BLACK , 0 , }	,
-					/* other */
-					{ "semi-condensed" , /* XXX This style is ignored. */
-						0 , 0 , } ,
-				} ;
-				float  size_f ;
-				
-				for( count = 0 ; count < sizeof(styles) / sizeof(styles[0]) ;
-					count ++)
-				{
-					size_t  len_v ;
-
-					len_v = strlen( styles[count].style) ;
-					
-					/* XXX strncasecmp is not portable? */
-					if( len >= len_v &&
-					    strncasecmp( p , styles[count].style , len_v) == 0)
-					{
-						*orig_p = '\0' ;
-						step = len_v ;
-						if( styles[count].weight)
-						{
-							*font_weight = styles[count].weight ;
-						}
-						else if( styles[count].slant)
-						{
-							*font_slant = styles[count].slant ;
-						}
-
-						goto  next_char ;
-					}
-				}
-				
-				if( *p != '0' &&	/* In case of "DevLys 010" font family. */
-				    sscanf( p , "%f" , &size_f) == 1)
-				{
-					/* If matched with %f, p has no more parameters. */
-
-					*orig_p = '\0' ;
-					*font_size = size_f ;
-
-					break ;
-				}
-				else
-				{
-					step = 1 ;
-				}
-			}
-		}
-		else
-		{
-			step = 1 ;
-		}
-
-next_char:
-		p += step ;
-		len -= step ;
-	}
-	
-	return  1 ;
-}
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 
 static u_int
-get_fc_col_width(
-	x_font_t *  font ,
-	double  fontsize_d ,
-	u_int  percent ,
-	u_int  letter_space
-	)
-{
-	if( percent > 0)
-	{
-		return  DIVIDE_ROUNDING(fontsize_d * font->cols * percent , 100 * 2) +
-			letter_space ;
-	}
-	else if( font->is_var_col_width)
-	{
-		return  0 ;
-	}
-	else if( strcmp( fc_size_type , FC_SIZE) == 0)
-	{
-		double  widthpix ;
-		double  widthmm ;
-		double  dpi ;
-
-		widthpix = DisplayWidth( font->display , DefaultScreen(font->display)) ;
-		widthmm = DisplayWidthMM( font->display , DefaultScreen(font->display)) ;
-
-		if( dpi_for_fc)
-		{
-			dpi = dpi_for_fc ;
-		}
-		else
-		{
-			dpi = (widthpix * 254) / (widthmm * 10) ;
-		}
-
-		return  DIVIDE_ROUNDINGUP(dpi * fontsize_d * font->cols , 72 * 2) + letter_space ;
-	}
-	else
-	{
-		return  DIVIDE_ROUNDINGUP(fontsize_d * font->cols , 2) + letter_space ;
-	}
-}
-
-static FcPattern *
-fc_pattern_create(
-	char *  family ,	/* can be NULL */
-	double  size ,
-	char *  encoding ,	/* can be NULL */
-	int  weight ,
-	int  slant ,
-	int  ch_width ,		/* can be 0 */
-	int  aa_opt
-	)
-{
-	FcPattern *  pattern ;
-
-	if( ! ( pattern = FcPatternCreate()))
-	{
-		return  NULL ;
-	}
-
-	if( family)
-	{
-		FcPatternAddString( pattern , FC_FAMILY , family) ;
-	}
-	FcPatternAddDouble( pattern , fc_size_type , size) ;
-	if( weight >= 0)
-	{
-		FcPatternAddInteger( pattern , FC_WEIGHT , weight) ;
-	}
-	if( slant >= 0)
-	{
-		FcPatternAddInteger( pattern , FC_SLANT , slant) ;
-	}
-	FcPatternAddInteger( pattern , FC_SPACING , ch_width > 0 ? FC_MONO : FC_PROPORTIONAL) ;
-	if( ch_width > 0)
-	{
-		/* XXX FC_CHAR_WIDTH doesn't make effect in cairo ... */
-		FcPatternAddInteger( pattern , FC_CHAR_WIDTH , ch_width) ;
-	}
-	if( aa_opt)
-	{
-		FcPatternAddBool( pattern , FC_ANTIALIAS , aa_opt == 1 ? True : False) ;
-	}
-	if( dpi_for_fc)
-	{
-		FcPatternAddDouble( pattern , FC_DPI , dpi_for_fc) ;
-	}
-#ifdef  USE_TYPE_XFT
-	if( encoding)
-	{
-		FcPatternAddString( pattern , XFT_ENCODING , encoding) ;
-	}
-#endif
-#if  0
-	FcPatternAddBool( pattern , "embeddedbitmap" , True) ;
-#endif
-
-	return  pattern ;
-}
-
-#endif	/* USE_TYPE_XFT / USE_TYPE_CAIRO */
-
-#ifdef  USE_TYPE_XFT
-
-static u_int
-xft_calculate_char_width(
-	Display *  display ,
-	XftFont *  xfont ,
-	const u_char *  ch ,		/* US-ASCII or Unicode */
-	size_t  len
-	)
-{
-	XGlyphInfo  extents ;
-
-	if( len == 1)
-	{
-		XftTextExtents8( display , xfont , ch , 1 , &extents) ;
-	}
-	else if( len == 2)
-	{
-		XftChar16  xch ;
-
-		xch = ((ch[0] << 8) & 0xff00) | (ch[1] & 0xff) ;
-
-		XftTextExtents16( display , xfont , &xch , 1 , &extents) ;
-	}
-	else if( len == 4)
-	{
-		XftChar32  xch ;
-
-		xch = ((ch[0] << 24) & 0xff000000) | ((ch[1] << 16) & 0xff0000) |
-			((ch[2] << 8) & 0xff00) | (ch[3] & 0xff) ;
-
-		XftTextExtents32( display , xfont , &xch , 1 , &extents) ;
-	}
-	else
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " char size %d is too large.\n" , len) ;
-	#endif
-
-		return  0 ;
-	}
-
-	return  extents.xOff ;
-}
-
-static XftFont *
-xft_font_open(
-	x_font_t *  font ,
-	char *  family ,	/* can be NULL */
-	double  size ,
-	char *  encoding ,	/* can be NULL */
-	int  weight ,
-	int  slant ,
-	int  ch_width ,
-	int  aa_opt
-	)
-{
-	FcPattern *  pattern ;
-	FcPattern *  match ;
-	FcResult  result ;
-	XftFont *  xfont ;
-
-	if( ! ( pattern = fc_pattern_create( family , size , encoding , weight , slant ,
-				ch_width , aa_opt)))
-	{
-		return  NULL ;
-	}
-
-	match = XftFontMatch( font->display , DefaultScreen( font->display) , pattern ,
-				&result) ;
-	FcPatternDestroy( pattern) ;
-	if( ! match)
-	{
-		return  NULL ;
-	}
-	
-#if  0
-	FcPatternPrint( match) ;
-#endif
-
-	if( ! ( xfont = XftFontOpenPattern( font->display , match)))
-	{
-		FcPatternDestroy( match) ;
-
-		return  NULL ;
-	}
-	
-	return  xfont ;
-}
-
-#endif
-
-#ifdef  USE_TYPE_CAIRO
-
-static u_int
-cairo_calculate_char_width(
-	cairo_scaled_font_t *  xfont ,
-	const u_char *  ch ,
-	size_t  len
-	)
-{
-	u_char  utf8[UTF_MAX_SIZE + 1] ;
-	cairo_text_extents_t  extents ;
-
-	utf8[ x_convert_ucs_to_utf8( utf8 , mkf_bytes_to_int( ch , len))] = '\0' ;
-	cairo_scaled_font_text_extents( xfont , utf8 , &extents) ;
-
-#if  0
-	kik_debug_printf( "%f %f %f\n" , extents.x_bearing , extents.width , extents.x_advance) ;
-#endif
-
-	return  DOUBLE_ROUNDUP_TO_INT(extents.x_advance) ;
-}
-
-static cairo_scaled_font_t *
-cairo_font_open(
-	x_font_t *  font ,
-	char *  family ,	/* can be NULL */
-	double  size ,
-	char *  encoding ,	/* can be NULL */
-	int  weight ,
-	int  slant ,
-	int  ch_width ,
-	int  aa_opt
-	)
-{
-	cairo_font_options_t *  options ;
-	cairo_t *  cairo ;
-	FcPattern *  pattern ;
-	FcPattern *  match ;
-	FcResult  result ;
-	cairo_font_face_t *  font_face ;
-	cairo_matrix_t  font_matrix ;
-	cairo_matrix_t  ctm ;
-	cairo_scaled_font_t *  xfont ;
-	double  pixel_size ;
-
-	if( ! ( pattern = fc_pattern_create( family , size , encoding , weight , slant ,
-				ch_width , aa_opt)))
-	{
-		return  NULL ;
-	}
-
-	FcConfigSubstitute( NULL , pattern , FcMatchPattern) ;
-
-	if( ! ( cairo = cairo_create( cairo_xlib_surface_create( font->display ,
-				DefaultRootWindow( font->display) ,
-				DefaultVisual( font->display , DefaultScreen( font->display)) ,
-				DisplayWidth( font->display , DefaultScreen( font->display)) ,
-				DisplayHeight( font->display , DefaultScreen( font->display))))))
-	{
-		FcPatternDestroy( pattern) ;
-
-		return  NULL ;
-	}
-
-	options = cairo_font_options_create() ;
-	cairo_get_font_options( cairo , options) ;
-	/* cairo_font_options_set_antialias( options , CAIRO_ANTIALIAS_GRAY) ; */
-	/* cairo_font_options_set_hint_style( options , CAIRO_HINT_STYLE_NONE) ; */
-	/* CAIRO_HINT_METRICS_ON disarranges column width by boldening etc. */
-	cairo_font_options_set_hint_metrics( options , CAIRO_HINT_METRICS_OFF) ;
-	cairo_ft_font_options_substitute( options , pattern) ;
-
-	FcDefaultSubstitute( pattern) ;
-
-	if( ! ( match = FcFontMatch( NULL , pattern , &result)))
-	{
-		cairo_destroy( cairo) ;
-		cairo_font_options_destroy( options) ;
-		FcPatternDestroy( pattern) ;
-
-		return  NULL ;
-	}
-
-#if  0
-	FcPatternPrint( match) ;
-#endif
-
-	font_face = cairo_ft_font_face_create_for_pattern( match) ;
-
-	FcPatternGetDouble( match , FC_PIXEL_SIZE , 0 , &pixel_size) ;
-	
-	cairo_matrix_init_identity( &font_matrix) ;
-	cairo_matrix_scale( &font_matrix , pixel_size , pixel_size) ;
-	cairo_get_matrix( cairo , &ctm) ;
-
-	xfont = cairo_scaled_font_create( font_face , &font_matrix , &ctm , options) ;
-
-	cairo_destroy( cairo) ;
-	cairo_font_options_destroy( options) ;
-	cairo_font_face_destroy( font_face) ;
-	FcPatternDestroy( pattern) ;
-	FcPatternDestroy( match) ;
-
-	if( cairo_scaled_font_status( xfont))
-	{
-		cairo_scaled_font_destroy( xfont) ;
-
-		return  NULL ;
-	}
-
-	return  xfont ;
-}
-
-#endif
-
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
-
-static void *
-ft_font_open(
-	x_font_t *  font ,
-	char *  family ,
-	double  size ,
-	char *  encoding ,
-	int  weight ,
-	int  slant ,
-	int  ch_width ,
-	int  aa_opt ,
-	int  use_xft
-	)
-{
-	if( use_xft)
-	{
-	#ifdef  USE_TYPE_XFT
-		return  xft_font_open( font , family , size , encoding , weight , slant ,
-				ch_width , aa_opt) ;
-	#endif
-	}
-	else
-	{
-	#ifdef  USE_TYPE_CAIRO
-		return  cairo_font_open( font , family , size , encoding , weight , slant ,
-				ch_width , aa_opt) ;
-	#endif
-	}
-
-	return  NULL ;
-}
-
-static int
-set_ft_font(
-	x_font_t *  font ,
-	const char *  fontname ,
-	u_int  fontsize ,
-	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
-	int  use_medium_for_bold ,	/* Not used for now. */
-	u_int  letter_space ,
-	int  aa_opt ,		/* 0 = default , 1 = enable , -1 = disable */
-	int  use_xft
-	)
-{
-	char *  font_encoding ;
-	int  weight ;
-	int  slant ;
-	u_int  ch_width ;
-	void *  xfont ;
-
-	/*
-	 * encoding, weight and slant can be modified in parse_fc_font_name().
-	 */
-	
-	font_encoding = NULL ;
-
-	if( font->id & FONT_BOLD)
-	{
-		weight = FC_WEIGHT_BOLD ;
-	}
-	else
-	{
-		weight = -1 ;	/* use default value */
-	}
-
-	slant = -1 ;	/* use default value */
-
-	if( fontname)
-	{
-		char *  p ;
-		char *  font_family ;
-		double  fontsize_d ;
-		u_int  percent ;
-
-		if( ( p = kik_str_alloca_dup( fontname)) == NULL)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
-		#endif
-
-			return  0 ;
-		}
-
-		fontsize_d = (double)fontsize ;
-		percent = 0 ;
-		if( parse_fc_font_name( &font_family , &weight , &slant , &fontsize_d ,
-						&font_encoding , &percent , p))
-		{
-			if( col_width == 0)
-			{
-				/* basic font (e.g. usascii) width */
-
-				ch_width = get_fc_col_width( font , fontsize_d , percent ,
-								letter_space) ;
-
-				if( font->is_vertical)
-				{
-					/*
-					 * !! Notice !!
-					 * The width of full and half character font is the same.
-					 */
-					ch_width *= 2 ;
-				}
-			}
-			else
-			{
-				if( font->is_var_col_width)
-				{
-					ch_width = 0 ;
-				}
-				else if( font->is_vertical)
-				{
-					/*
-					 * !! Notice !!
-					 * The width of full and half character font is the same.
-					 */
-					ch_width = col_width ;
-				}
-				else
-				{
-					ch_width = col_width * font->cols ;
-				}
-			}
-
-		#ifdef  DEBUG
-			kik_debug_printf( "Loading font %s%s%s %f %d\n" , font_family ,
-				weight == XFT_WEIGHT_BOLD ? ":Bold" :
-					weight == XFT_WEIGHT_LIGHT ? " Light" : "" ,
-				slant == XFT_SLANT_ITALIC ? ":Italic" : "" ,
-				fontsize_d , font->is_var_col_width) ;
-		#endif
-
-			if( ( xfont = ft_font_open( font , font_family , fontsize_d ,
-					font_encoding , weight , slant , ch_width ,
-					aa_opt , use_xft)))
-			{
-				goto  font_found ;
-			}
-		}
-
-		kik_warn_printf( "Font %s (for size %f) couldn't be loaded.\n" ,
-			fontname , fontsize_d) ;
-	}
-
-	if( col_width == 0)
-	{
-		/* basic font (e.g. usascii) width */
-
-		ch_width = get_fc_col_width( font , (double)fontsize , 0 , letter_space) ;
-	
-		if( font->is_vertical)
-		{
-			/*
-			 * !! Notice !!
-			 * The width of full and half character font is the same.
-			 */
-			ch_width *= 2 ;
-		}
-	}
-	else
-	{
-		if( font->is_var_col_width)
-		{
-			ch_width = 0 ;
-		}
-		else if( font->is_vertical)
-		{
-			/*
-			 * !! Notice !!
-			 * The width of full and half character font is the same.
-			 */
-			ch_width = col_width ;
-		}
-		else
-		{
-			ch_width = col_width * font->cols ;
-		}
-	}
-
-	if( ( xfont = ft_font_open( font , NULL , (double)fontsize , font_encoding ,
-				weight , slant , ch_width , aa_opt , use_xft)))
-	{
-		goto  font_found ;
-	}
-
-#ifdef  DEBUG
-	kik_warn_printf( KIK_DEBUG_TAG " ft_font_open(%s) failed.\n" , fontname) ;
-#endif
-
-	return  0 ;
-
-font_found:
-
-#if  defined(FC_EMBOLDEN) /* Synthetic emboldening (fontconfig >= 2.3.0) */ \
-	|| defined(USE_TYPE_CAIRO)
-	font->is_double_drawing = 0 ;
-#else
-	if( weight == FC_WEIGHT_BOLD &&
-	    XftPatternGetInteger( xfont->pattern , FC_WEIGHT , 0 , &weight) == XftResultMatch &&
-	    weight != FC_WEIGHT_BOLD)
-	{
-		font->is_double_drawing = 1 ;
-	}
-	else
-	{
-		font->is_double_drawing = 0 ;
-	}
-#endif
-
-	font->x_off = 0 ;
-
-	font->is_proportional = font->is_var_col_width ;
-
-	if( use_xft)
-	{
-	#ifdef  USE_TYPE_XFT
-		font->xft_font = xfont ;
-
-		font->height = font->xft_font->height ;
-		font->height_to_baseline = font->xft_font->ascent ;
-
-		if( ch_width == 0)
-		{
-			/*
-			 * Proportional (font->is_var_col_width is true)
-			 * letter_space is ignored in variable column width mode.
-			 */
-			font->width = xft_calculate_char_width( font->display , xfont , "W" , 1) ;
-		}
-		else
-		{
-			font->width = ch_width ;
-		}
-	#endif	/* USE_TYPE_XFT */
-	}
-	else
-	{
-	#ifdef  USE_TYPE_CAIRO
-		cairo_font_extents_t  extents ;
-
-		font->cairo_font = xfont ;
-
-		cairo_scaled_font_extents( font->cairo_font , &extents) ;
-		font->height = DOUBLE_ROUNDUP_TO_INT(extents.height) ;
-		font->height_to_baseline = DOUBLE_ROUNDUP_TO_INT(extents.ascent) ;
-
-		/* XXX letter_space is always ignored. */
-		if( font->cols == 2)
-		{
-			font->width = DOUBLE_ROUNDUP_TO_INT(extents.max_x_advance) ;
-		}
-		else
-		{
-			font->width = cairo_calculate_char_width( font->cairo_font , "W" , 1) ;
-		}
-
-	#if  0
-		if( col_width > 0 /* is not usascii */ && ! font->is_proportional &&
-		    ch_width != font->width)
-		{
-			kik_warn_printf( "Font width(%d) is not matched with "
-				"standard width(%d).\n" , font->width , ch_width) ;
-
-			font->is_proportional = 1 ;
-
-			if( font->width < ch_width)
-			{
-				font->x_off = (ch_width - font->width) / 2 ;
-			}
-
-			font->width = ch_width ;
-		}
-	#endif
-	#endif	/* USE_TYPE_CAIRO */
-	}
-
-	/*
-	 * checking if font height/height_to_baseline member is sane.
-	 * font width must be always sane.
-	 */
-
-	if( font->height == 0)
-	{
-		/* XXX this may be inaccurate. */
-		font->height = fontsize ;
-	}
-
-	if( font->height_to_baseline == 0)
-	{
-		/* XXX this may be inaccurate. */
-		font->height_to_baseline = fontsize ;
-	}
-
-	/*
-	 * set_decsp_font() is called after dummy font is loaded to get font metrics.
-	 * Since dummy font encoding is "iso8859-1", loading rarely fails.
-	 */
-	/* XXX dec specials must always be composed for now */
-	if( /* compose_dec_special_font && */ FONT_CS(font->id) == DEC_SPECIAL)
-	{
-		return  set_decsp_font( font) ;
-	}
-
-	return  1 ;
-}
-
-#endif  /* USE_TYPE_XFT */
-
-#ifdef  USE_TYPE_XCORE
-
-static u_int
-calculate_char_width(
+xcore_calculate_char_width(
 	Display *  display ,
 	XFontStruct *  xfont ,
 	const u_char *  ch ,
@@ -1206,7 +408,7 @@ load_xfont(
 }
 
 static int
-set_xfont(
+xcore_set_font(
 	x_font_t *  font ,
 	const char *  fontname ,
 	u_int  fontsize ,
@@ -1274,7 +476,7 @@ set_xfont(
 				goto  font_found ;
 			}
 
-			kik_warn_printf( "Font %s couldn't be loaded.\n" , font_xlfd) ;
+			kik_msg_printf( "Font %s couldn't be loaded.\n" , font_xlfd) ;
 		}
 	}
 
@@ -1415,8 +617,8 @@ font_found:
 		u_int  w_width ;
 		u_int  i_width ;
 
-		w_width = calculate_char_width( font->display , font->xfont , "W" , 1) ;
-		i_width = calculate_char_width( font->display , font->xfont , "i" , 1) ;
+		w_width = xcore_calculate_char_width( font->display , font->xfont , "W" , 1) ;
+		i_width = xcore_calculate_char_width( font->display , font->xfont , "i" , 1) ;
 
 		if( w_width == 0)
 		{
@@ -1567,9 +769,9 @@ font_found:
 		{
 			if( font->width != col_width * font->cols)
 			{
-				kik_warn_printf( "Font width(%d) is not matched with "
+				kik_msg_printf( "Font(id %x) width(%d) is not matched with "
 					"standard width(%d).\n" ,
-					font->width , col_width * font->cols) ;
+					font->id , font->width , col_width * font->cols) ;
 
 				font->is_proportional = 1 ;
 
@@ -1624,7 +826,101 @@ font_found:
 	return  1 ;
 }
 
-#endif	/* USE_TYPE_XCORE */
+#endif
+
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE)
+
+static u_int
+xft_calculate_char_width(
+	x_font_t *  font ,
+	const u_char *  ch ,		/* US-ASCII or Unicode */
+	size_t  len
+	)
+{
+	int (*func)( x_font_t * , const u_char * , size_t) ;
+
+	if( ! ( func = x_load_type_xft_func( X_CALCULATE_CHAR_WIDTH)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font , ch , len) ;
+}
+
+static int
+xft_set_font(
+	x_font_t *  font ,
+	const char *  fontname ,
+	u_int  fontsize ,
+	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
+	int  use_medium_for_bold ,	/* Not used for now. */
+	u_int  letter_space ,
+	int  aa_opt ,		/* 0 = default , 1 = enable , -1 = disable */
+	int  use_point_size_for_fc ,
+	double  dpi_for_fc
+	)
+{
+	int (*func)( x_font_t * , const char * , u_int , u_int , int , u_int , int ,
+			int , double) ;
+
+	if( ! ( func = x_load_type_xft_func( X_SET_FONT)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font , fontname , fontsize , col_width , use_medium_for_bold ,
+			letter_space , aa_opt , use_point_size_for_fc , dpi_for_fc) ;
+}
+
+#endif
+
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE)
+
+static u_int
+cairo_calculate_char_width(
+	x_font_t *  font ,
+	const u_char *  ch ,		/* US-ASCII or Unicode */
+	size_t  len
+	)
+{
+	int (*func)( x_font_t * , const u_char * , size_t) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_CALCULATE_CHAR_WIDTH)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font , ch , len) ;
+}
+
+static int
+cairo_set_font(
+	x_font_t *  font ,
+	const char *  fontname ,
+	u_int  fontsize ,
+	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
+	int  use_medium_for_bold ,	/* Not used for now. */
+	u_int  letter_space ,
+	int  aa_opt ,		/* 0 = default , 1 = enable , -1 = disable */
+	int  use_point_size_for_fc ,
+	double  dpi_for_fc
+	)
+{
+	int (*func)( x_font_t * , const char * , u_int , u_int , int , u_int , int ,
+			int , double) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_SET_FONT)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( font , fontname , fontsize , col_width , use_medium_for_bold ,
+			letter_space , aa_opt , use_point_size_for_fc , dpi_for_fc) ;
+}
+
+#endif
 
 
 /* --- global functions --- */
@@ -1691,13 +987,13 @@ x_font_new(
 		font->is_vertical = 0 ;
 	}
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	font->xfont = NULL ;
 #endif
-#ifdef  USE_TYPE_XFT
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	font->xft_font = NULL ;
 #endif
-#ifdef  USE_TYPE_CAIRO
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	font->cairo_font = NULL ;
 #endif
 	font->decsp_font = NULL ;
@@ -1707,14 +1003,13 @@ x_font_new(
 	default:
 		return  NULL ;
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	case  TYPE_XFT:
-	case  TYPE_CAIRO:
-		if( ! set_ft_font( font , fontname , fontsize , col_width , use_medium_for_bold ,
-				letter_space ,
+		if( ! xft_set_font( font , fontname , fontsize , col_width ,
+				use_medium_for_bold , letter_space ,
 				(font_present & FONT_AA) == FONT_AA ?
 					1 : ((font_present & FONT_NOAA) == FONT_NOAA ? -1 : 0) ,
-				type_engine == TYPE_XFT))
+				use_point_size_for_fc , dpi_for_fc))
 		{
 			free( font) ;
 
@@ -1724,13 +1019,29 @@ x_font_new(
 		break ;
 #endif
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	case  TYPE_CAIRO:
+		if( ! cairo_set_font( font , fontname , fontsize , col_width ,
+				use_medium_for_bold , letter_space ,
+				(font_present & FONT_AA) == FONT_AA ?
+					1 : ((font_present & FONT_NOAA) == FONT_NOAA ? -1 : 0) ,
+				use_point_size_for_fc , dpi_for_fc))
+		{
+			free( font) ;
+
+			return  NULL ;
+		}
+
+		break ;
+#endif
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	case  TYPE_XCORE:
 		if( font_present & FONT_AA)
 		{
 			return  NULL ;
 		}
-		else if( ! set_xfont( font , fontname , fontsize , col_width ,
+		else if( ! xcore_set_font( font , fontname , fontsize , col_width ,
 				use_medium_for_bold , letter_space))
 		{
 			free( font) ;
@@ -1738,13 +1049,26 @@ x_font_new(
 			return  NULL ;
 		}
 
-		break ;
+		goto  end ;
 #endif
 	}
 
+	/*
+	 * set_decsp_font() is called after dummy xft/cairo font is loaded to get font metrics.
+	 * Since dummy font encoding is "iso8859-1", loading rarely fails.
+	 */
+	/* XXX dec specials must always be composed for now */
+	if( /* compose_dec_special_font && */ FONT_CS(font->id) == DEC_SPECIAL)
+	{
+		set_decsp_font( font) ;
+	}
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
+end:
+#endif
 	if( font->is_proportional && ! font->is_var_col_width)
 	{
-		kik_warn_printf(
+		kik_msg_printf(
 			"Characters (cs %d) are drawn *one by one* to arrange column width.\n" ,
 			FONT_CS(font->id)) ;
 	}
@@ -1761,21 +1085,21 @@ x_font_delete(
 	x_font_t *  font
 	)
 {
-#ifdef  USE_TYPE_XFT
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	if( font->xft_font)
 	{
-		XftFontClose( font->display , font->xft_font) ;
-		font->xft_font = NULL ;
+		xft_unset_font( font) ;
 	}
 #endif
-#ifdef  USE_TYPE_CAIRO
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	if( font->cairo_font)
 	{
-		cairo_scaled_font_destroy( font->cairo_font) ;
-		font->cairo_font = NULL ;
+		cairo_unset_font( font) ;
 	}
 #endif
-#ifdef  USE_TYPE_XCORE
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	if( font->xfont)
 	{
 		XFreeFont( font->display , font->xfont) ;
@@ -1829,7 +1153,7 @@ x_calculate_char_width(
 {
 	if( font->is_var_col_width && font->is_proportional && ! font->decsp_font)
 	{
-	#ifdef  USE_TYPE_XFT
+	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 		if( font->xft_font)
 		{
 			u_char  ucs4[4] ;
@@ -1845,11 +1169,11 @@ x_calculate_char_width(
 				len = 4 ;
 			}
 
-			return  xft_calculate_char_width( font->display , font->xft_font ,
-								ch , len) ;
+			return  xft_calculate_char_width( font , ch , len) ;
 		}
 	#endif
-	#ifdef  USE_TYPE_CAIRO
+
+	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 		if( font->cairo_font)
 		{
 			u_char  ucs4[4] ;
@@ -1865,17 +1189,21 @@ x_calculate_char_width(
 				len = 4 ;
 			}
 
-			return  cairo_calculate_char_width( font->cairo_font , ch , len) ;
-		}
-	#endif
-	#ifdef  USE_TYPE_XCORE
-		if( font->xfont)
-		{
-			return  calculate_char_width( font->display , font->xfont , ch , len) ;
+			return  cairo_calculate_char_width( font , ch , len) ;
 		}
 	#endif
 
-		kik_error_printf( KIK_DEBUG_TAG " couldn't calculate correct font width.\n") ;
+	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
+		if( font->xfont)
+		{
+			return  xcore_calculate_char_width( font->display , font->xfont ,
+					ch , len) ;
+		}
+	#endif
+
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " couldn't calculate correct font width.\n") ;
+	#endif
 	}
 
 	return  font->width ;
@@ -1904,16 +1232,7 @@ x_font_use_point_size_for_fc(
 	int  bool
 	)
 {
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
-	if( bool)
-	{
-		fc_size_type = FC_SIZE ;
-	}
-	else
-	{
-		fc_size_type = FC_PIXEL_SIZE ;
-	}
-#endif
+	use_point_size_for_fc = bool ;
 }
 
 void
@@ -1921,12 +1240,10 @@ x_font_set_dpi_for_fc(
 	double  dpi
 	)
 {
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	dpi_for_fc = dpi ;
-#endif
 }
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 
 static int  use_cp932_ucs_for_xft = 0 ;
 
@@ -2057,71 +1374,7 @@ no_diff:
 	return  convert_to_ucs4( ucs4_bytes , src_bytes , src_size , cs) ;
 }
 
-size_t
-x_convert_ucs_to_utf8(
-	u_char *  utf8 ,	/* size of utf8 should be greater than 7. */
-	u_int32_t  ucs
-	)
-{
-	/* ucs is unsigned */
-	if( /* 0x00 <= ucs && */ ucs <= 0x7f)
-	{
-		*utf8 = ucs ;
-
-		return  1 ;
-	}
-	else if( ucs <= 0x07ff)
-	{
-		*(utf8 ++) = ((ucs >> 6) & 0xff) | 0xc0 ;
-		*utf8 = (ucs & 0x3f) | 0x80 ;
-
-		return  2 ;
-	}
-	else if( ucs <= 0xffff)
-	{
-		*(utf8 ++) = ((ucs >> 12) & 0x0f) | 0xe0 ;
-		*(utf8 ++) = ((ucs >> 6) & 0x3f) | 0x80 ;
-		*utf8 = (ucs & 0x3f) | 0x80 ;
-
-		return  3 ;
-	}
-	else if( ucs <= 0x1fffff)
-	{
-		*(utf8 ++) = ((ucs >> 18) & 0x07) | 0xf0 ;
-		*(utf8 ++) = ((ucs >> 12) & 0x3f) | 0x80 ;
-		*(utf8 ++) = ((ucs >> 6) & 0x3f) | 0x80 ;
-		*utf8 = (ucs & 0x3f) | 0x80 ;
-
-		return  4 ;
-	}
-	else if( ucs <= 0x03ffffff)
-	{
-		*(utf8 ++) = ((ucs >> 24) & 0x03) | 0xf8 ;
-		*(utf8 ++) = ((ucs >> 18) & 0x3f) | 0x80 ;
-		*(utf8 ++) = ((ucs >> 12) & 0x3f) | 0x80 ;
-		*(utf8 ++) = ((ucs >> 6) & 0x3f) | 0x80 ;
-		*utf8 = (ucs & 0x3f) | 0x80 ;
-
-		return  5 ;
-	}
-	else if( ucs <= 0x7fffffff)
-	{
-		*(utf8 ++) = ((ucs >> 30) & 0x01) | 0xfc ;
-		*(utf8 ++) = ((ucs >> 24) & 0x3f) | 0x80 ;
-		*(utf8 ++) = ((ucs >> 18) & 0x3f) | 0x80 ;
-		*(utf8 ++) = ((ucs >> 12) & 0x3f) | 0x80 ;
-		*(utf8 ++) = ((ucs >> 6) & 0x3f) | 0x80 ;
-		*utf8 = (ucs & 0x3f) | 0x80 ;
-
-		return  6 ;
-	}
-	else
-	{
-		return  0 ;
-	}
-}
-
-#endif	/* USE_TYPE_XFT / USE_TYPE_CAIRO */
+#endif
 
 
 #ifdef  DEBUG
@@ -2131,13 +1384,13 @@ x_font_dump(
 	x_font_t *  font
 	)
 {
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	kik_msg_printf( "Font id %x: XFont %p" , font->id , font->xfont) ;
 #endif
-#ifdef  USE_TYPE_XFT
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	kik_msg_printf( "Font id %x: XftFont %p" , font->id , font->xft_font) ;
 #endif
-#ifdef  USE_TYPE_CAIRO
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	kik_msg_printf( "Font id %x: CairoFont %p" , font->id , font->cairo_font) ;
 #endif
 
