@@ -71,9 +71,8 @@ static wincs_info_t  wincs_info_table[] =
 static cs_info_t  cs_info_table[] =
 {
 	{ ISO10646_UCS4_1 , DEFAULT_CHARSET , } ,
-	{ ISO10646_UCS2_1 , DEFAULT_CHARSET , } ,
 	
-	{ DEC_SPECIAL , DEFAULT_CHARSET , } ,
+	{ DEC_SPECIAL , SYMBOL_CHARSET , } ,
 	{ ISO8859_1_R , ANSI_CHARSET , } ,
 	{ ISO8859_2_R , DEFAULT_CHARSET , } ,
 	{ ISO8859_3_R , EASTEUROPE_CHARSET , } ,
@@ -454,8 +453,12 @@ x_font_new(
 	font_family = NULL ;
 	percent = 0 ;
 	fontsize_d = (double)fontsize ;
-	
-	if( fontname)
+
+	if( FONT_CS(font->id) == DEC_SPECIAL)
+	{
+		font_family = "Tera Special" ;
+	}
+	else if( fontname)
 	{
 		char *  p ;
 
@@ -495,9 +498,11 @@ x_font_new(
 
 	if( ! font->fid)
 	{
+	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " CreateFont failed.\n") ;
 		free( font) ;
-	
+	#endif
+
 		return  NULL ;
 	}
 	else
@@ -533,7 +538,11 @@ x_font_new(
 	#ifdef  ENABLE_PROPORTIONAL
 		if( font->cols == 2)
 		{
-			/* XXX */
+			/*
+			 * XXX
+			 * MaxCharWidth of MS Gothic is AveCharWidth * 2 + 1.
+			 *                                               ^^^
+			 */
 			font->width = tm.tmMaxCharWidth ;
 		}
 		else
@@ -655,7 +664,7 @@ x_font_new(
 		{
 			if( font->width != col_width * font->cols)
 			{
-				kik_warn_printf( "Font width(%d) is not matched with "
+				kik_msg_printf( "Font width(%d) is not matched with "
 					"standard width(%d).\n" ,
 					font->width , col_width * font->cols) ;
 
@@ -672,18 +681,19 @@ x_font_new(
 	}
 
 
-	if( wincsinfo->cs == ANSI_CHARSET || FONT_CS(font->id) == ISO10646_UCS4_1)
+	if( wincsinfo->cs == ANSI_CHARSET || wincsinfo->cs == SYMBOL_CHARSET ||
+	    FONT_CS(font->id) == ISO10646_UCS4_1)
 	{
 		font->conv = NULL ;
 	}
 	else
 	{
-		font->conv = ml_conv_new( wincsinfo->encoding) ;
-
-		if( ! font->conv)
+		if( ! ( font->conv = ml_conv_new( wincsinfo->encoding)))
 		{
+		#ifdef  DEBUG
 			kik_warn_printf( KIK_DEBUG_TAG " ml_conv_new(font id %x) failed.\n" ,
 				font->id) ;
+		#endif
 		}
 	}
 	
@@ -691,7 +701,7 @@ x_font_new(
 
 	if( font->is_proportional && ! font->is_var_col_width)
 	{
-		kik_warn_printf(
+		kik_msg_printf(
 			"Characters (cs %d) are drawn *one by one* to arrange column width.\n" ,
 			FONT_CS(font->id)) ;
 
