@@ -864,10 +864,8 @@ window_realized(
 
 	cand_screen = (x_im_candidate_screen_t*) win ;
 
-	x_window_set_use_xft( &cand_screen->window ,
-		x_get_type_engine( cand_screen->font_man) == TYPE_XFT) ;
-	x_window_set_use_cairo( &cand_screen->window ,
-		x_get_type_engine( cand_screen->font_man) == TYPE_CAIRO) ;
+	x_window_set_type_engine( &cand_screen->window ,
+		x_get_type_engine( cand_screen->font_man)) ;
 
 	x_window_set_fg_color( win , x_get_xcolor( cand_screen->color_man , ML_FG_COLOR)) ;
 	x_window_set_bg_color( win , x_get_xcolor( cand_screen->color_man , ML_BG_COLOR)) ;
@@ -875,14 +873,6 @@ window_realized(
 	x_window_set_override_redirect( &cand_screen->window , 1) ;
 }
 
-static void
-window_finalized(
-	x_window_t *  win
-	)
-{
-	/* do nothing */
-}
-
 static void
 window_exposed(
 	x_window_t *  win ,
@@ -956,15 +946,6 @@ button_pressed(
 
 }
 
-static void
-button_released(
-	x_window_t *  win ,
-	XButtonEvent *  event
-	)
-{
-	/* do nothing */
-}
-
 
 /* --- global functions --- */
 
@@ -1032,11 +1013,15 @@ x_im_candidate_screen_new(
 
 	/* callbacks for window events */
 	cand_screen->window.window_realized = window_realized ;
+#if  0
 	cand_screen->window.window_finalized = window_finalized ;
+#endif
 	cand_screen->window.window_exposed = window_exposed ;
 	cand_screen->window.window_focused = window_focused ;
 	cand_screen->window.window_unfocused = window_unfocused ;
+#if  0
 	cand_screen->window.button_released = button_released ;
+#endif
 	cand_screen->window.button_pressed = button_pressed ;
 #if 0
 	cand_screen->window.button_press_continued = button_press_continued ;
