@@ -18,6 +18,7 @@ x_prepare_for_main_config(
 {
 	char *  rcpath ;
 
+#ifdef  ENABLE_BACKWARD_COMPAT
 	/*
 	 * XXX
 	 * "mlterm/core" is for backward compatibility with 1.9.44
@@ -34,6 +35,7 @@ x_prepare_for_main_config(
 		kik_conf_read( conf , rcpath) ;
 		free( rcpath) ;
 	}
+#endif
 
 	if( ( rcpath = kik_get_sys_rc_path( "mlterm/main")))
 	{
@@ -58,13 +60,15 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , '&' , "borderless" , 1 , "borderless" ,
 		"override redirect [false]") ;
 	kik_conf_add_opt( conf , '*' , "type" , 0 , "type_engine" ,
-	#if  defined(USE_TYPE_XCORE)
-		"type engine [xcore]") ;
-	#elif  defined(USE_TYPE_XFT)
-		"type engine [xft]") ;
-	#elif  defined(USE_TYPE_CAIRO)
-		"type engine [cairo]") ;
+		"type engine "
+	#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
+		"[xcore]"
+	#elif  ! defined(USE_TYPE_XFT) && defined(USE_TYPE_CAIRO)
+		"[cairo]"
+	#else
+		"[xft]"
 	#endif
+		) ;
 #endif	/* USE_WIN32GUI */
 
 	kik_conf_add_opt( conf , '1' , "wscr" , 0 , "screen_width_ratio" ,
@@ -87,17 +91,17 @@ x_prepare_for_main_config(
 		"cursor foreground color") ;
 	kik_conf_add_opt( conf , '0' , "crbg" , 0 , "cursor_bg_color" ,
 		"cursor background color") ;
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	kik_conf_add_opt( conf , 'A' , "aa" , 1 , "use_anti_alias" , 
 		"forcibly use anti alias font by using Xft or cairo") ;
 #endif
 	kik_conf_add_opt( conf , 'B' , "sbbg" , 0 , "sb_bg_color" , 
 		"scrollbar background color") ;
-#ifdef  USE_IND
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
 	kik_conf_add_opt( conf , 'C' , "ind" , 1 , "use_ind" , 
-		"use indic (ligature text) [true(if use_bidi is false)]") ;
+		"use indic (ligature text) [false]") ;
 #endif
-#ifdef  USE_FRIBIDI
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	kik_conf_add_opt( conf , 'D' , "bi" , 1 , "use_bidi" , 
 		"use bidi (bi-directional text) [true]") ;
 #endif
@@ -183,7 +187,7 @@ x_prepare_for_main_config(
 		"step in changing font size in GUI configurator [1]") ;
 	kik_conf_add_opt( conf , '\0' , "iconpath" , 0 , "icon_path" ,
 		"path to an imagefile to be use as an window icon") ;
-#ifdef  USE_FRIBIDI
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	kik_conf_add_opt( conf , '\0' , "bimode" , 0 , "bidi_mode" ,
 		"bidi mode [normal]") ;
 #endif
@@ -197,11 +201,11 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , '\0' , "serv" , 0 , "default_server" ,
 		"connecting server by default") ;
 #endif
-#if  defined(USE_WIN32API) && defined(USE_LIBSSH2)
+#ifdef  USE_LIBSSH2
+	#ifdef  USE_WIN32API
 	kik_conf_add_opt( conf , '\0' , "nodialog" , 1 , "skip_dialog" ,
 		"if possible, skip dialog to input server address, password and so on") ;
-#endif
-#ifdef  USE_LIBSSH2
+	#endif
 	kik_conf_add_opt( conf , '\0' , "pubkey" , 0 , "ssh_public_key" ,
 		"ssh public key file "
 	#ifdef  USE_WIN32API
@@ -222,7 +226,9 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , '\0' , "csp" , 0 , "letter_space" ,
 		"extra space between letters in pixels [0]") ;
 	kik_conf_add_opt( conf , '\0' , "ucsprop" , 1 , "use_unicode_property" ,
-		"use unicode property for characters") ;
+		"use unicode property for characters [false]") ;
+	kik_conf_add_opt( conf , '\0' , "logmsg" , 1 , "logging_msg" ,
+		"output messages to ~/.mlterm/msg[pid].log [true]") ;
 	kik_conf_set_end_opt( conf , 'e' , NULL , "exec_cmd" , 
 		"execute external command") ;
 
@@ -240,6 +246,16 @@ x_main_config_init(
 	char *  value ;
 	char *  invalid_msg = "%s %s is not valid.\n" ;
 
+	if( ( value = kik_conf_get_value( conf , "logging_msg")) &&
+	    strcmp( value , "false") == 0)
+	{
+		kik_set_msg_log_file_name( NULL) ;
+	}
+	else
+	{
+		kik_set_msg_log_file_name( "mlterm/msg") ;
+	}
+
 #ifndef  USE_WIN32GUI
 	if( ( value = kik_conf_get_value( conf , "display")) == NULL)
 #endif
@@ -361,16 +377,6 @@ x_main_config_init(
 		}
 	}
 
-	main_config->font_present = 0 ;
-
-	if( ( value = kik_conf_get_value( conf , "use_variable_column_width")))
-	{
-		if( strcmp( value , "true") == 0)
-		{
-			main_config->font_present |= FONT_VAR_WIDTH ;
-		}
-	}
-
 	main_config->step_in_changing_font_size = 1 ;
 	
 	if( ( value = kik_conf_get_value( conf , "step_in_changing_font_size")))
@@ -387,9 +393,9 @@ x_main_config_init(
 		}
 	}
 
-#if  defined(USE_TYPE_XCORE)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	main_config->type_engine = TYPE_XCORE ;
-#elif  defined(USE_TYPE_XFT)
+#elif   defined(USE_TYPE_XFT)
 	main_config->type_engine = TYPE_XFT ;
 #else
 	main_config->type_engine = TYPE_CAIRO ;
@@ -400,7 +406,9 @@ x_main_config_init(
 		main_config->type_engine = x_get_type_engine_by_name( value) ;
 	}
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+	main_config->font_present = 0 ;
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 	if( ( value = kik_conf_get_value( conf , "use_anti_alias")))
 	{
 		if( strcmp( value , "true") == 0)
@@ -409,10 +417,10 @@ x_main_config_init(
 			if( main_config->type_engine == TYPE_XCORE)
 			{
 				/* forcibly use xft or cairo */
-			#ifdef  USE_TYPE_XFT
-				main_config->type_engine = TYPE_XFT ;
-			#else
+			#if  ! defined(USE_TYPE_XFT) && defined(USE_TYPE_CAIRO)
 				main_config->type_engine = TYPE_CAIRO ;
+			#else
+				main_config->type_engine = TYPE_XFT ;
 			#endif
 			}
 		}
@@ -423,18 +431,21 @@ x_main_config_init(
 	}
 #endif
 
+	if( ( value = kik_conf_get_value( conf , "use_variable_column_width")))
+	{
+		if( strcmp( value , "true") == 0)
+		{
+			main_config->font_present |= FONT_VAR_WIDTH ;
+		}
+	}
+
 	main_config->vertical_mode = 0 ;
 	
 	if( ( value = kik_conf_get_value( conf , "vertical_mode")))
 	{
 		if( ( main_config->vertical_mode = ml_get_vertical_mode( value)))
 		{
-			/*
-			 * vertical font is automatically used under vertical mode.
-			 * similler processing is done in x_screen.c:change_vertical_mode.
-			 */
 			main_config->font_present |= FONT_VERTICAL ;
-			main_config->font_present &= ~FONT_VAR_WIDTH ;
 		}
 	}
 
@@ -895,7 +906,7 @@ x_main_config_init(
 		main_config->encoding = ML_ISO8859_1 ;
 	}
 
-#ifdef  USE_FRIBIDI
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	main_config->use_bidi = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "use_bidi")))
@@ -905,28 +916,30 @@ x_main_config_init(
 			main_config->use_bidi = 0 ;
 		}
 	}
+#else
+	main_config->use_bidi = 0 ;
+#endif
 
 	main_config->bidi_mode = BIDI_NORMAL_MODE ;
 	
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_FRIBIDI)
 	if( ( value = kik_conf_get_value( conf , "bidi_mode")))
 	{
 		main_config->bidi_mode = ml_get_bidi_mode( value) ;
 	}
-#else
-	main_config->use_bidi = 0 ;
-	main_config->bidi_mode = BIDI_NORMAL_MODE ;
 #endif
 
-#ifdef  USE_IND
-	main_config->use_ind = 1 ;
+	main_config->use_ind = 0 ;
 
+#if  ! defined(NO_DYNAMIC_LOAD_CTL) || defined(USE_IND)
 	if( ( value = kik_conf_get_value( conf , "use_ind")))
 	{
-		if( strcmp( value , "false") == 0)
+		if( strcmp( value , "true") == 0)
 		{
-			main_config->use_ind = 0 ;
+			main_config->use_ind = 1 ;
 		}
 	}
+#endif
 
 	/* If value is "none" or not is also checked in x_screen.c */
 	if( ( value = kik_conf_get_value( conf , "mod_meta_key")) &&
@@ -938,9 +951,6 @@ x_main_config_init(
 	{
 		main_config->mod_meta_key = NULL ;
 	}
-#else
-	main_config->use_ind = 0 ;
-#endif
 	
 	if( ( value = kik_conf_get_value( conf , "mod_meta_mode")))
 	{
@@ -1224,6 +1234,8 @@ x_main_config_final(
 	free( main_config->input_method) ;
 	free( main_config->init_str) ;
 
+	kik_set_msg_log_file_name(NULL) ;
+
 #if  defined(USE_WIN32API) || defined(USE_LIBSSH2)
 	if( main_config->server_list)
 	{
