@@ -199,30 +199,6 @@ utf_selection_notified(
 		buf , len) ;
 }
 
-#ifdef  USE_WIN32GUI
-static void
-button_pressed(
-	x_window_t *  win ,
-	XButtonEvent *  event ,
-	int  click_num
-	)
-{
-	x_sb_screen_t *  sb_screen ;
-
-	sb_screen = (x_sb_screen_t*) win ;
-
-	/*
-	 * dispatch to screen for WM_MOUSEWHEEL which is delivered to root window.
-	 *
-	 * XXX Incomplete
-	 * event->x and event->y should be modified according to scrollbar position,
-	 * 
-	 */
-	(*sb_screen->screen->window.button_pressed)( &sb_screen->screen->window ,
-			event , click_num) ;
-}
-#endif
-
 static void
 xct_selection_notified(
 	x_window_t *  win ,
@@ -778,9 +754,6 @@ x_sb_screen_new(
 	sb_screen->window.child_window_resized = child_window_resized ;
 	sb_screen->window.window_exposed = window_exposed ;
 	sb_screen->window.key_pressed = key_pressed ;
-#ifdef  USE_WIN32GUI
-	sb_screen->window.button_pressed = button_pressed ;
-#endif
 	sb_screen->window.utf_selection_notified = utf_selection_notified ;
 	sb_screen->window.xct_selection_notified = xct_selection_notified ;
 	sb_screen->window.window_deleted = window_deleted ;
