@@ -246,6 +246,7 @@ read_conf(
 
 	while( kik_conf_io_read( from , &key , &value))
 	{
+	#ifdef  ENABLE_BACKWARD_COMPAT
 		/*
 		 * XIM_OPEN and XIM_CLOSE are removed.
 		 */
@@ -253,9 +254,10 @@ read_conf(
 		    strcmp( value, "XIM_CLOSE") == 0)
 		{
 			/* This warning will be removed. */
-			kik_warn_printf( "%s in %s is no longer supported\n",
+			kik_msg_printf( "%s in %s is no longer supported\n",
 					 value , filename);
 		}
+	#endif
 
 		/*
 		 * [shortcut key]=[operation]
