@@ -14,18 +14,16 @@
 #include  <string.h>		/* memset/memcpy */
 #include  <X11/Xutil.h>		/* for XSizeHints */
 #include  <X11/Xatom.h>
-#ifdef  USE_TYPE_CAIRO
-#include  <cairo/cairo-xlib.h>
-#endif
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>	/* realloc/free */
 
 #include  "x_xic.h"
 #include  "x_picture.h"
-
 #ifndef  DISABLE_XDND
 #include  "x_dnd.h"
 #endif
+#include  "x_type_loader.h"
+
 
 /*
  * Atom macros.
@@ -595,6 +593,166 @@ reset_client_leader(
 	return  leader ;
 }
 
+static void
+convert_to_decsp_font_index(
+	u_char *  str ,
+	u_int  len
+	)
+{
+	while( len != 0)
+	{
+		if( *str == 0x5f)
+		{
+			*str = 0x7f ;
+		}
+		else if( 0x5f < *str && *str < 0x7f)
+		{
+			(*str) -= 0x5f ;
+		}
+
+		len -- ;
+		str ++ ;
+	}
+}
+
+
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE)
+
+static int
+x_window_set_use_xft(
+	x_window_t *  win ,
+	int  use_xft
+	)
+{
+	int (*func)( x_window_t * , int) ;
+
+	if( ! ( func = x_load_type_xft_func( X_WINDOW_SET_TYPE)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win , use_xft) ;
+}
+
+static int
+x_window_xft_draw_string8(
+	x_window_t *  win ,
+	x_font_t *  font ,
+	x_color_t *  fg_color ,
+	int  x ,
+	int  y ,
+	u_char *  str ,
+	size_t  len
+	)
+{
+	int (*func)( x_window_t * , x_font_t * , x_color_t * , int , int , u_char * , size_t) ;
+
+	if( ! ( func = x_load_type_xft_func( X_WINDOW_DRAW_STRING8)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win , font , fg_color , x , y , str , len) ;
+}
+
+static int
+x_window_xft_draw_string32(
+	x_window_t *  win ,
+	x_font_t *  font ,
+	x_color_t *  fg_color ,
+	int  x ,
+	int  y ,
+	/* FcChar32 */ u_int32_t *  str ,
+	size_t  len
+	)
+{
+	int (*func)( x_window_t * , x_font_t * , x_color_t * , int , int ,
+		/* FcChar32 */ u_int32_t * , size_t) ;
+
+	if( ! ( func = x_load_type_xft_func( X_WINDOW_DRAW_STRING32)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win , font , fg_color , x , y , str , len) ;
+}
+
+static int
+x_window_set_use_cairo(
+	x_window_t *  win ,
+	int  use_cairo
+	)
+{
+	int (*func)( x_window_t * , int) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_WINDOW_SET_TYPE)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win , use_cairo) ;
+}
+
+static int
+x_window_cairo_draw_string8(
+	x_window_t *  win ,
+	x_font_t *  font ,
+	x_color_t *  fg_color ,
+	int  x ,
+	int  y ,
+	u_char *  str ,
+	size_t  len
+	)
+{
+	int (*func)( x_window_t * , x_font_t * , x_color_t * , int , int , u_char * , size_t) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_WINDOW_DRAW_STRING8)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win , font , fg_color , x , y , str , len) ;
+}
+
+static int
+x_window_cairo_draw_string32(
+	x_window_t *  win ,
+	x_font_t *  font ,
+	x_color_t *  fg_color ,
+	int  x ,
+	int  y ,
+	/* FcChar32 */ u_int32_t *  str ,
+	size_t  len
+	)
+{
+	int (*func)( x_window_t * , x_font_t * , x_color_t * , int , int ,
+		/* FcChar32 */ u_int32_t * , size_t) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_WINDOW_DRAW_STRING32)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win , font , fg_color , x , y , str , len) ;
+}
+
+static int
+cairo_resize(
+	x_window_t *  win
+	)
+{
+	int (*func)( x_window_t *) ;
+
+	if( ! ( func = x_load_type_cairo_func( X_WINDOW_RESIZE)))
+	{
+		return  0 ;
+	}
+
+	return  (*func)( win) ;
+}
+
+#endif /* NO_DYNAMIC_LOAD_TYPE */
+
 
 /* --- global functions --- */
 
@@ -621,8 +779,12 @@ x_window_init(
 	
 	win->my_window = None ;
 
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	win->xft_draw = NULL ;
+#endif
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	win->cairo_draw = NULL ;
+#endif
 
 	win->gc = NULL ;
 
@@ -691,6 +853,7 @@ x_window_init(
 	win->window_focused = NULL ;
 	win->window_unfocused = NULL ;
 	win->key_pressed = NULL ;
+	win->pointer_motion = NULL ;
 	win->button_motion = NULL ;
 	win->button_released = NULL ;
 	win->button_pressed = NULL ;
@@ -732,9 +895,7 @@ x_window_final(
 	
 	x_xic_deactivate( win) ;
 
-#ifdef  USE_TYPE_XFT
-	x_window_set_use_xft( win , 0) ;
-#endif
+	x_window_set_type_engine( win , TYPE_XCORE) ;
 
 	if( win->create_gc)
 	{
@@ -755,66 +916,26 @@ x_window_final(
  * Call this function in window_realized event at first.
  */
 int
-x_window_set_use_xft(
+x_window_set_type_engine(
 	x_window_t *  win ,
-	int  use_xft
+	x_type_engine_t  type_engine
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( use_xft && ! win->xft_draw)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
+	if( ( win->xft_draw != NULL) != ( type_engine == TYPE_XFT))
 	{
-		if( ( win->xft_draw = XftDrawCreate( win->disp->display , win->my_window ,
-					win->disp->visual , win->disp->colormap)))
-		{
-			x_window_set_use_cairo( win , 0) ;
-		}
-
-		return  1 ;
-	}
-	else if( ! use_xft && win->xft_draw)
-	{
-		XftDrawDestroy( win->xft_draw) ;
-		win->xft_draw = NULL ;
-
-		return  1 ;
+		x_window_set_use_xft( win , ( type_engine == TYPE_XFT)) ;
 	}
 #endif
 
-	return  0 ;
-}
-
-/*
- * Call this function in window_realized event at first.
- */
-int
-x_window_set_use_cairo(
-	x_window_t *  win ,
-	int  use_cairo
-	)
-{
-#ifdef  USE_TYPE_CAIRO
-	if( use_cairo && ! win->cairo_draw)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
+	if( ( win->cairo_draw != NULL) != ( type_engine == TYPE_CAIRO))
 	{
-		if( ( win->cairo_draw = cairo_create( cairo_xlib_surface_create(
-						win->disp->display , win->my_window ,
-						win->disp->visual ,
-						ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)))))
-		{
-			x_window_set_use_xft( win , 0) ;
-		}
-
-		return  1 ;
-	}
-	else if( ! use_cairo && win->cairo_draw)
-	{
-		cairo_destroy( win->cairo_draw) ;
-		win->cairo_draw = NULL ;
-
-		return  1 ;
+		x_window_set_use_cairo( win , ( type_engine == TYPE_CAIRO)) ;
 	}
 #endif
 
-	return  0 ;
+	return  1 ;
 }
 
 int
@@ -1480,11 +1601,10 @@ x_window_resize(
 		(*win->window_resized)( win) ;
 	}
 
-#ifdef  USE_TYPE_CAIRO
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	if( win->cairo_draw)
 	{
-		cairo_xlib_surface_set_size( cairo_get_target( win->cairo_draw) ,
-			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+		cairo_resize( win) ;
 	}
 #endif
 
@@ -1654,6 +1774,9 @@ x_window_move(
 	int  y
 	)
 {
+	win->x = x ;
+	win->y = y ;
+
 	XMoveWindow( win->disp->display , win->my_window , x , y) ;
 
 	return  1 ;
@@ -1668,18 +1791,8 @@ x_window_clear(
 	u_int	height
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( ! win->wall_picture_is_set && win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
-			x + win->margin , y + win->margin , width , height) ;
-	}
-	else
-#endif
-	{
-		XClearArea( win->disp->display , win->my_window ,
-			x + win->margin , y + win->margin , width , height , False) ;
-	}
+	XClearArea( win->disp->display , win->my_window ,
+		x + win->margin , y + win->margin , width , height , False) ;
 
 	return  1 ;
 }
@@ -1694,31 +1807,14 @@ x_window_clear_margin_area(
 		return  1 ;
 	}
 
-#ifdef  USE_TYPE_XFT
-	if( ! win->wall_picture_is_set && win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
-			0 , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
-			win->margin , 0 , win->width , win->margin) ;
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
-			win->width + win->margin , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
-			win->margin , win->height + win->margin ,
-			win->width , win->margin) ;
-	}
-	else
-#endif
-	{
-		XClearArea( win->disp->display , win->my_window ,
-			0 , 0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
-		XClearArea( win->disp->display , win->my_window ,
-			win->margin , 0 , win->width , win->margin , 0) ;
-		XClearArea( win->disp->display , win->my_window , win->width + win->margin ,
-			0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
-		XClearArea( win->disp->display , win->my_window , win->margin ,
-			win->height + win->margin , win->width , win->margin , 0) ;
-	}
+	XClearArea( win->disp->display , win->my_window ,
+		0 , 0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
+	XClearArea( win->disp->display , win->my_window ,
+		win->margin , 0 , win->width , win->margin , 0) ;
+	XClearArea( win->disp->display , win->my_window , win->width + win->margin ,
+		0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
+	XClearArea( win->disp->display , win->my_window , win->margin ,
+		win->height + win->margin , win->width , win->margin , 0) ;
 
 	return  1 ;
 }
@@ -1728,18 +1824,8 @@ x_window_clear_all(
 	x_window_t *  win
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( ! win->wall_picture_is_set && win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
-			0 , 0 , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
-	}
-	else
-#endif
-	{
-		XClearArea( win->disp->display , win->my_window , 0 , 0 ,
-			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) , 0) ;
-	}
+	XClearArea( win->disp->display , win->my_window , 0 , 0 ,
+		ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) , 0) ;
 
 	return  1 ;
 }
@@ -1753,20 +1839,10 @@ x_window_fill(
 	u_int	height
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
-			x + win->margin , y + win->margin , width , height) ;
-	}
-	else
-#endif
-	{
-		restore_fg_color( win) ;
+	restore_fg_color( win) ;
 
-		XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
-			x + win->margin , y + win->margin , width , height) ;
-	}
+	XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
+		x + win->margin , y + win->margin , width , height) ;
 
 	return  1 ;
 }
@@ -1781,20 +1857,10 @@ x_window_fill_with(
 	u_int	height
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( color) ,
-			x + win->margin , y + win->margin , width , height) ;
-	}
-	else
-#endif
-	{
-		x_gc_set_fg_color( win->gc, color->pixel) ;
+	x_gc_set_fg_color( win->gc, color->pixel) ;
 
-		XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
-			x + win->margin , y + win->margin , width , height) ;
-	}
+	XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
+		x + win->margin , y + win->margin , width , height) ;
 
 	return  1 ;
 }
@@ -1804,20 +1870,10 @@ x_window_blank(
 	x_window_t *  win
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) , 0 , 0 ,
-			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
-	}
-	else
-#endif
-	{
-		restore_fg_color( win) ;
+	restore_fg_color( win) ;
 
-		XFillRectangle( win->disp->display , win->my_window , win->gc->gc , 0 , 0 ,
-			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
-	}
+	XFillRectangle( win->disp->display , win->my_window , win->gc->gc , 0 , 0 ,
+		ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 
 	return  1 ;
 }
@@ -2007,7 +2063,7 @@ x_window_receive_event(
 	else if( event->type == MotionNotify)
 	{
 		XEvent  ahead ;
-
+		
 		while( XEventsQueued(win->disp->display , QueuedAfterReading))
 		{
 			XPeekEvent( win->disp->display , &ahead) ;
@@ -2043,9 +2099,12 @@ x_window_receive_event(
 			win->prev_button_press_event.y = event->xmotion.y ;
 			win->prev_button_press_event.time = event->xmotion.time ;
 		}
-		else
+		else if( win->pointer_motion)
 		{
-			/* (*win->pointer_motion)( ... ) */
+			event->xmotion.x -= win->margin ;
+			event->xmotion.y -= win->margin ;
+
+			(*win->pointer_motion)( win , &event->xmotion) ;
 		}
 	}
 	else if( event->type == ButtonRelease)
@@ -2239,11 +2298,10 @@ x_window_receive_event(
 
 			is_changed = 1 ;
 
-		#ifdef  USE_TYPE_CAIRO
+		#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 			if( win->cairo_draw)
 			{
-				cairo_xlib_surface_set_size( cairo_get_target( win->cairo_draw) ,
-					ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+				cairo_resize( win) ;
 			}
 		#endif
 		}
@@ -2804,6 +2862,8 @@ x_window_draw_decsp_string(
 	u_int  len
 	)
 {
+	convert_to_decsp_font_index( str , len) ;
+
 	if( font->decsp_font)
 	{
 		x_gc_set_fg_color( win->gc, fg_color->pixel) ;
@@ -2812,7 +2872,7 @@ x_window_draw_decsp_string(
 				win->my_window , win->gc->gc , x + win->margin , y + win->margin ,
 				str , len) ;
 	}
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	else if( font->xfont)
 	{
 		return  x_window_draw_string( win , font , fg_color , x , y , str , len) ;
@@ -2836,6 +2896,8 @@ x_window_draw_decsp_image_string(
 	u_int  len
 	)
 {
+	convert_to_decsp_font_index( str , len) ;
+
 	if( font->decsp_font)
 	{
 		x_gc_set_fg_color( win->gc, fg_color->pixel) ;
@@ -2845,7 +2907,7 @@ x_window_draw_decsp_image_string(
 				win->my_window , win->gc->gc , x + win->margin , y + win->margin ,
 				str , len) ;
 	}
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 	else if( font->xfont)
 	{
 		return  x_window_draw_image_string( win , font , fg_color , bg_color ,
@@ -2858,7 +2920,7 @@ x_window_draw_decsp_image_string(
 	}
 }
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 int
 x_window_draw_string(
 	x_window_t *  win ,
@@ -2976,7 +3038,7 @@ x_window_draw_image_string16(
 }
 #endif
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 int
 x_window_ft_draw_string8(
 	x_window_t *  win ,
@@ -2988,58 +3050,23 @@ x_window_ft_draw_string8(
 	size_t  len
 	)
 {
-#ifdef  USE_TYPE_CAIRO
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	if( win->cairo_draw)
 	{
-		u_char *  buf ;
-
-		if( ! ( buf = alloca( len + 1)))
-		{
-			return  0 ;
-		}
-
-		memcpy( buf , str , len) ;
-		buf[len] = '\0' ;
-
-		cairo_set_antialias( win->cairo_draw , CAIRO_ANTIALIAS_NONE) ;
-		cairo_set_scaled_font( win->cairo_draw , font->cairo_font) ;
-		cairo_set_source_rgb( win->cairo_draw ,
-			(double)fg_color->red / 255.0 , (double)fg_color->green / 255.0 ,
-			(double)fg_color->blue / 255.0) ;
-		cairo_move_to( win->cairo_draw , x + win->margin , y + win->margin) ;
-		cairo_show_text( win->cairo_draw , buf) ;
-
-		if( font->is_double_drawing)
-		{
-			cairo_move_to( win->cairo_draw , x + win->margin + 1 , y + win->margin) ;
-			cairo_show_text( win->cairo_draw , str) ;
-		}
+		return  x_window_cairo_draw_string8( win , font , fg_color , x , y , str , len) ;
 	}
 	else
 #endif
-#ifdef  USE_TYPE_XFT
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	if( win->xft_draw)
 	{
-		XftColor *  xftcolor ;
-
-		xftcolor = x_color_to_xft( fg_color) ;
-
-		XftDrawString8( win->xft_draw , xftcolor , font->xft_font ,
-			x + win->margin , y + win->margin , str , len) ;
-
-		if( font->is_double_drawing)
-		{
-			XftDrawString8( win->xft_draw , xftcolor , font->xft_font ,
-				x + win->margin + 1 , y + win->margin , str , len) ;
-		}
+		return  x_window_xft_draw_string8( win , font , fg_color , x , y , str , len) ;
 	}
 	else
 #endif
 	{
 		return  0 ;
 	}
-
-	return  1 ;
 }
 
 int
@@ -3049,69 +3076,32 @@ x_window_ft_draw_string32(
 	x_color_t *  fg_color ,
 	int  x ,
 	int  y ,
-	FcChar32 *  str ,
+	/* FcChar32 */ u_int32_t *  str ,
 	u_int  len
 	)
 {
-#ifdef  USE_TYPE_CAIRO
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_CAIRO)
 	if( win->cairo_draw)
 	{
-		u_char *  buf ;
-		u_int  count ;
-		char *  p ;
-
-		if( ! ( p = buf = alloca( 8 * len + 1)))
-		{
-			return  0 ;
-		}
-
-		for( count = 0 ; count < len ; count++)
-		{
-			p += x_convert_ucs_to_utf8( p , str[count]) ;
-		}
-		*p = '\0' ;
-
-		cairo_set_scaled_font( win->cairo_draw , font->cairo_font) ;
-		cairo_set_source_rgb( win->cairo_draw ,
-			(double)fg_color->red / 255.0 , (double)fg_color->green / 255.0 ,
-			(double)fg_color->blue / 255.0) ;
-		cairo_move_to( win->cairo_draw , x + win->margin , y + win->margin) ;
-		cairo_show_text( win->cairo_draw , buf) ;
-
-		if( font->is_double_drawing)
-		{
-			cairo_move_to( win->cairo_draw , x + win->margin + 1 , y + win->margin) ;
-			cairo_show_text( win->cairo_draw , buf) ;
-		}
+		return  x_window_cairo_draw_string32( win , font , fg_color , x , y , str , len) ;
 	}
 	else
 #endif
-#ifdef  USE_TYPE_XFT
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT)
 	if( win->xft_draw)
 	{
-		XftColor *  xftcolor ;
-
-		xftcolor = x_color_to_xft( fg_color) ;
-
-		XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
-			x + win->margin , y + win->margin , str , len) ;
-
-		if( font->is_double_drawing)
-		{
-			XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
-				x + win->margin + 1 , y + win->margin , str , len) ;
-		}
+		return  x_window_xft_draw_string32( win , font , fg_color , x , y , str , len) ;
 	}
 	else
 #endif
 	{
 		return  0 ;
 	}
-
-	return  1 ;
 }
+
 #endif
 
+
 int
 x_window_draw_rect_frame(
 	x_window_t *  win ,
@@ -3121,35 +3111,19 @@ x_window_draw_rect_frame(
 	int  y2
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( win->xft_draw)
+	XPoint  points[5] =
 	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
-			x1 , y1 , 1 , y2 - y1 + 1) ;
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
-			x1 , y1 , x2 - x1 + 1 , 1) ;
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
-			x1 , y2 , x2 - x1 + 1 , 1) ;
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
-			x2 , y1 , 1 , y2 - y1 + 1) ;
-	}
-	else
-#endif
-	{
-		XPoint  points[5] =
-		{
-			{ x1 , y1 } ,
-			{ x1 , y2 } ,
-			{ x2 , y2 } ,
-			{ x2 , y1 } ,
-			{ x1 , y1 } ,
-		} ;
+		{ x1 , y1 } ,
+		{ x1 , y2 } ,
+		{ x2 , y2 } ,
+		{ x2 , y1 } ,
+		{ x1 , y1 } ,
+	} ;
 
-		restore_fg_color(win) ;
+	restore_fg_color(win) ;
 
-		XDrawLines( win->disp->display , win->my_window , win->gc->gc , points , 5 ,
-			CoordModeOrigin) ;
-	}
+	XDrawLines( win->disp->display , win->my_window , win->gc->gc , points , 5 ,
+		CoordModeOrigin) ;
 
 	return  1 ;
 }
@@ -3163,19 +3137,9 @@ x_window_draw_line(
 	int  y2
 	)
 {
-#ifdef  USE_TYPE_XFT
-	if( win->xft_draw)
-	{
-		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
-			x1 , y1 , x2 - x1 + 1 , y2 - y1 + 1) ;
-	}
-	else
-#endif
-	{
-		restore_fg_color(win) ;
+	restore_fg_color(win) ;
 
-		XDrawLine( win->disp->display , win->my_window , win->gc->gc , x1 , y1 , x2 , y2) ;
-	}
+	XDrawLine( win->disp->display , win->my_window , win->gc->gc , x1 , y1 , x2 , y2) ;
 
 	return  1 ;
 }
