@@ -12,15 +12,6 @@
 #include  <X11/cursorfont.h>	/* for cursor shape */
 #endif
 
-#ifdef  USE_TYPE_XFT
-#include  <X11/Xft/Xft.h>
-#endif
-
-#ifdef  USE_TYPE_CAIRO
-#include  <cairo/cairo.h>
-#include  <fontconfig/fontconfig.h>
-#endif
-
 #include  <kiklib/kik_types.h>
 #include  <mkf/mkf_parser.h>
 #include  <ml_char.h>
@@ -34,6 +25,14 @@
 #define  ACTUAL_WIDTH(win)  ((win)->width + (win)->margin * 2)
 #define  ACTUAL_HEIGHT(win)  ((win)->height + (win)->margin * 2)
 
+/*
+ * Don't use win->parent in xlib to check if win is root window or not
+ * because mlterm can work as libvte.
+ *   vte window
+ *      |
+ *   mlterm window ... x_window_t::parent == NULL
+ *                     x_window_t::parent_window == vte window
+ */
 #define  PARENT_WINDOWID_IS_TOP(win)  ((win)->parent_window == (win)->disp->my_window)
 
 
@@ -69,6 +68,8 @@ typedef struct x_dnd_context *  x_dnd_context_ptr_t ;
 /* Defined in x_picture.h */
 typedef struct x_picture_modifier *  x_picture_modifier_ptr_t ;
 typedef struct x_icon_picture *  x_icon_picture_ptr_t ;
+typedef struct _XftDraw *  xft_draw_ptr_t ;
+typedef struct _cairo *  cairo_ptr_t ;
 
 typedef struct  x_window
 {
@@ -76,16 +77,12 @@ typedef struct  x_window
 	
 	Window  my_window ;
 
-#ifdef  USE_TYPE_XFT
-	XftDraw *  xft_draw ;
-#else
-	void *  xft_draw ;		/* dummy to keep the size of x_window_t */
-#endif
-#ifdef  USE_TYPE_CAIRO
-	cairo_t *  cairo_draw ;
-#else
-	void *  cairo_draw ;		/* dummy to keep the size of x_window_t */
-#endif
+	/*
+	 * Don't remove if USE_XFT and USE_CAIRO are not defined to keep the size of x_window_t
+	 * for x_im_xxx_screen_t.
+	 */
+	xft_draw_ptr_t  xft_draw ;
+	cairo_ptr_t  cairo_draw ;
 
 	x_color_t  fg_color ;
 	x_color_t  bg_color ;
@@ -135,11 +132,11 @@ typedef struct  x_window
 	/*
 	 * XDND
 	 */
-#ifndef  DISABLE_XDND
+	/*
+	 * Don't remove if DISABLE_XDND is defined to keep the size of x_window_t for
+	 * x_im_xxx_screen_t.
+	 */
 	x_dnd_context_ptr_t  dnd ;
-#else
-	void *  dnd ; /* dummy to keep the size of x_window_t */
-#endif
 
 	/*
 	 * XClassHint
@@ -172,6 +169,7 @@ typedef struct  x_window
 	void (*window_focused)( struct x_window *) ;
 	void (*window_unfocused)( struct x_window *) ;
 	void (*key_pressed)( struct x_window * , XKeyEvent *) ;
+	void (*pointer_motion)( struct x_window * , XMotionEvent *) ;
 	void (*button_motion)( struct x_window * , XMotionEvent *) ;
 	void (*button_released)( struct x_window * , XButtonEvent *) ;
 	void (*button_pressed)( struct x_window * , XButtonEvent * , int) ;
@@ -183,11 +181,11 @@ typedef struct  x_window
 	void (*utf_selection_requested)( struct x_window * , XSelectionRequestEvent * , Atom) ;
 	void (*xct_selection_notified)( struct x_window * , u_char * , size_t) ;
 	void (*utf_selection_notified)( struct x_window * , u_char * , size_t) ;
-#ifndef  DISABLE_XDND
+	/*
+	 * Don't remove if DISABLE_XDND is defined to keep the size of x_window_t
+	 * for x_im_xxx_screen_t.
+	 */
 	void (*set_xdnd_config)( struct x_window * , char * ,  char * , char * ) ;
-#else
-	void *  set_xdnd_config ; /* dummy to the keep size of x_window_t */
-#endif
 	void (*window_deleted)( struct x_window *) ;
 	void (*mapping_notify)( struct x_window *) ;
 
@@ -201,9 +199,7 @@ int  x_window_init( x_window_t *  win ,
 
 int  x_window_final( x_window_t *  win) ;
 
-int  x_window_set_use_xft( x_window_t *  win , int  use_xft) ;
-
-int  x_window_set_use_cairo( x_window_t *  win , int  use_cairo) ;
+int  x_window_set_type_engine( x_window_t *  win , x_type_engine_t  type_engine) ;
 
 int  x_window_init_event_mask( x_window_t *  win , long  event_mask) ;
 
@@ -319,7 +315,7 @@ int  x_window_draw_decsp_image_string( x_window_t *  win , x_font_t *  font ,
  * Use x_draw_str* functions usually.
  */
 
-#ifdef  USE_TYPE_XCORE
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 int  x_window_draw_string( x_window_t *  win , x_font_t *  font , x_color_t *  fg_color ,
 	int  x , int  y , u_char *  str , u_int  len) ;
 
@@ -335,12 +331,12 @@ int  x_window_draw_image_string16( x_window_t *  win , x_font_t *  font ,
 	XChar2b *  str , u_int  len) ;
 #endif
 
-#if  defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
+#if  ! defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XFT) || defined(USE_TYPE_CAIRO)
 int  x_window_ft_draw_string8( x_window_t *  win , x_font_t *  font ,
 	x_color_t *  fg_color , int  x , int  y , u_char *  str , size_t  len) ;
 
 int  x_window_ft_draw_string32( x_window_t *  win , x_font_t *  font ,
-	x_color_t *  fg_color , int  x , int  y , FcChar32 *  str , u_int  len) ;
+	x_color_t *  fg_color , int  x , int  y , /* FcChar32 */ u_int32_t *  str , u_int  len) ;
 #endif
 
 int  x_window_draw_rect_frame( x_window_t *  win , int  x1 , int  y1 , int  x2 , int  y2) ;
