@@ -24,15 +24,22 @@
 
 #define  WM_APP_PAINT  (WM_APP + 0x0)
 
-/* ACTUAL_WIDTH is the width of client area, ACTUAL_WINDOW_WIDTH is that of window area. */
+/*
+ * ACTUAL_(WIDTH|HEIGHT) is the width of client area, ACTUAL_WINDOW_(WIDTH|HEIGHT) is
+ * that of window area.
+ */
+#if  1
+#define  ACTUAL_WINDOW_WIDTH(win) (ACTUAL_WIDTH(win) + decorate_width)
+#define  ACTUAL_WINDOW_HEIGHT(win) (ACTUAL_HEIGHT(win) + decorate_height)
+#else
 #define  ACTUAL_WINDOW_WIDTH(win) \
 	ACTUAL_WIDTH(win) + GetSystemMetrics(SM_CXEDGE) * 2 + \
 	/* GetSystemMetrics(SM_CXBORDER) * 2 + */ GetSystemMetrics(SM_CXFRAME)
-/* ACTUAL_HEIGHT is the height of client area, ACTUAL_WINDOW_HEIGHT is that of window area. */
 #define  ACTUAL_WINDOW_HEIGHT(win) \
 	ACTUAL_HEIGHT(win) + GetSystemMetrics(SM_CYEDGE) * 2 + \
 	/* GetSystemMetrics(SM_CXBORDER) * 2 + */ GetSystemMetrics(SM_CYFRAME) + \
 	GetSystemMetrics(SM_CYCAPTION)
+#endif
 
 
 #if  0
@@ -44,6 +51,8 @@
 
 static int  click_interval = 250 ;	/* millisecond, same as xterm. */
 static mkf_parser_t *  m_cp_parser ;
+static LONG  decorate_width ;
+static LONG  decorate_height ;		/* Height of Title bar etc. */
 
 
 /* --- static functions --- */
@@ -769,6 +778,48 @@ draw_string(
 	return  1 ;
 }
 
+/*
+ * Return 1 => decorate size is changed.
+ *        0 => decorate size is not changed.
+ */
+static int
+update_decorate_size(
+	x_window_t *  win
+	)
+{
+	RECT  wr ;
+	RECT  cr ;
+	LONG  width ;
+	LONG  height ;
+
+	if( win->parent)
+	{
+		return  0 ;
+	}
+	
+	GetWindowRect( win->my_window , &wr) ;
+	GetClientRect( win->my_window , &cr) ;
+	width = wr.right - wr.left - cr.right + cr.left ;
+	height = wr.bottom - wr.top - cr.bottom + cr.top ;
+
+#ifdef  DEBUG
+	kik_debug_printf( KIK_DEBUG_TAG " DECORATE W %d H %d -> W %d H %d\n" ,
+		decorate_width , decorate_height , width , height) ;
+#endif
+
+	if( width != decorate_width || height != decorate_height)
+	{
+		decorate_width = width ;
+		decorate_height = height ;
+
+		return  1 ;
+	}
+	else
+	{
+		return  0 ;
+	}
+}
+
 
 /* --- global functions --- */
 
@@ -853,11 +904,7 @@ x_window_init(
 	win->click_num = 0 ;
 	win->button_is_pressing = 0 ;
 	win->dnd = NULL ;
-#ifndef  UTF16_IME_CHAR
-	win->app_name = "mlterm" ;
-#else
-	win->app_name = L"mlterm" ;
-#endif
+	win->app_name = __("mlterm") ;
 
 	win->window_realized = NULL ;
 	win->window_finalized = NULL ;
@@ -866,6 +913,7 @@ x_window_init(
 	win->window_focused = NULL ;
 	win->window_unfocused = NULL ;
 	win->key_pressed = NULL ;
+	win->pointer_motion = NULL ;
 	win->button_motion = NULL ;
 	win->button_released = NULL ;
 	win->button_pressed = NULL ;
@@ -930,18 +978,9 @@ x_window_final(
 }
 
 int
-x_window_set_use_xft(
+x_window_set_type_engine(
 	x_window_t *  win ,
-	int  use_xft
-	)
-{
-	return  0 ;
-}
-
-int
-x_window_set_use_cairo(
-	x_window_t *  win ,
-	int  use_cairo
+	x_type_engine_t  type_engine
 	)
 {
 	return  0 ;
@@ -1237,21 +1276,12 @@ x_window_show(
 		GetSystemMetrics(SM_CYFRAME), GetSystemMetrics(SM_CYCAPTION)) ;
 #endif
 
-#ifndef  UTF16_IME_CHAR
-	win->my_window = CreateWindowEx( 0 , "MLTERM" , win->app_name ,
-#else
-	win->my_window = CreateWindowExW( 0 , L"MLTERM" , win->app_name ,
-#endif
-				PARENT_WINDOWID_IS_TOP(win) ?
-					WS_OVERLAPPEDWINDOW : WS_CHILD | WS_VISIBLE ,
-				PARENT_WINDOWID_IS_TOP(win) ?
-					CW_USEDEFAULT : win->x ,
-				PARENT_WINDOWID_IS_TOP(win) ?
-					CW_USEDEFAULT : win->y ,
-				PARENT_WINDOWID_IS_TOP(win) ?
-					ACTUAL_WINDOW_WIDTH(win) : ACTUAL_WIDTH(win) ,
-				PARENT_WINDOWID_IS_TOP(win) ?
-					ACTUAL_WINDOW_HEIGHT(win) : ACTUAL_HEIGHT(win) ,
+	win->my_window = CreateWindowEx( 0 , __("MLTERM") , win->app_name ,
+				! win->parent ? WS_OVERLAPPEDWINDOW : WS_CHILD | WS_VISIBLE ,
+				! win->parent ? CW_USEDEFAULT : win->x ,
+				! win->parent ? CW_USEDEFAULT : win->y ,
+				! win->parent ? ACTUAL_WINDOW_WIDTH(win) : ACTUAL_WIDTH(win) ,
+				! win->parent ? ACTUAL_WINDOW_HEIGHT(win) : ACTUAL_HEIGHT(win) ,
 				win->parent_window , NULL , win->disp->display->hinst , NULL) ;
 
   	if( ! win->my_window)
@@ -1283,6 +1313,14 @@ x_window_show(
 	 * really visualized.
 	 */
 
+	/* Don't place this before x_window_show(children). */
+	if( update_decorate_size( win))
+	{
+		SetWindowPos( win->my_window , 0 , 0 , 0 ,
+			ACTUAL_WINDOW_WIDTH(win) , ACTUAL_WINDOW_HEIGHT(win) ,
+			SWP_NOMOVE | SWP_NOZORDER) ;
+	}
+
 	if( win->is_mapped)
 	{
 		ShowWindow( win->my_window , SW_SHOWNORMAL) ;
@@ -1356,9 +1394,10 @@ x_window_resize(
 		(*win->parent->child_window_resized)( win->parent , win) ;
 	}
 
-	SetWindowPos( win->my_window, 0, 0, 0,
-		win->parent ? ACTUAL_WIDTH(win) : ACTUAL_WINDOW_WIDTH(win),
-		win->parent ? ACTUAL_HEIGHT(win) : ACTUAL_WINDOW_HEIGHT(win),
+	update_decorate_size( win) ;
+	SetWindowPos( win->my_window , 0 , 0 , 0 ,
+		win->parent ? ACTUAL_WIDTH(win) : ACTUAL_WINDOW_WIDTH(win) ,
+		win->parent ? ACTUAL_HEIGHT(win) : ACTUAL_WINDOW_HEIGHT(win) ,
 		SWP_NOMOVE | SWP_NOZORDER) ;
 		
 	if( (flag & NOTIFY_TO_MYSELF) && win->window_resized)
@@ -1431,8 +1470,14 @@ x_window_move(
 	int  y
 	)
 {
-	SetWindowPos( win->my_window, 0, x, y,
-		ACTUAL_WINDOW_WIDTH(win), ACTUAL_WINDOW_HEIGHT(win),
+	win->x = x ;
+	win->y = y ;
+
+	update_decorate_size( win) ;
+
+	SetWindowPos( win->my_window , 0 , x , y ,
+		win->parent ? ACTUAL_WIDTH(win) : ACTUAL_WINDOW_WIDTH(win) ,
+		win->parent ? ACTUAL_HEIGHT(win) : ACTUAL_WINDOW_HEIGHT(win) ,
 		SWP_NOSIZE | SWP_NOZORDER) ;
 
 	return  1 ;
@@ -1955,22 +2000,21 @@ x_window_receive_event(
 
 			bev.time = GetMessageTime() ;
 
-			bev.x = LOWORD(event->lparam) - win->margin ;
-			bev.y = HIWORD(event->lparam) - win->margin ;
-
 			bev.state = 0 ;
+		#if  0
 			if( event->wparam & MK_LBUTTON)
 			{
 				bev.state |= Button1Mask ;
 			}
-			if( event->wparam & MK_RBUTTON)
+			if( event->wparam & MK_MBUTTON)
 			{
 				bev.state |= Button2Mask ;
 			}
-			if( event->wparam & MK_MBUTTON)
+			if( event->wparam & MK_RBUTTON)
 			{
 				bev.state |= Button3Mask ;
 			}
+		#endif
 			if( event->wparam & MK_SHIFT)
 			{
 				bev.state |= ShiftMask ;
@@ -1982,6 +2026,43 @@ x_window_receive_event(
 
 			if( event->msg == WM_MOUSEWHEEL)
 			{
+				/* Note that WM_MOUSEWHEEL event is reported to top window. */
+
+				POINT  p ;
+				u_int  count ;
+				
+				p.x = LOWORD(event->lparam) ;
+				p.y = HIWORD(event->lparam) ;
+
+				ScreenToClient( win->my_window , &p) ;
+
+				/*
+				 * XXX
+				 * Children of more than 1 generation are not considered.
+				 *      top window
+				 *       /       \
+				 *     win1      win2
+				 *     /  \      /   \
+				 *   win3 win4 win5 win6 <= Not considered.
+				 */
+				for( count = 0 ; count < win->num_of_children ; count++)
+				{
+					if( win->children[count]->x <= p.x &&
+					    p.x <= win->children[count]->x +
+					           win->children[count]->width &&
+					    win->children[count]->y <= p.y &&
+					    p.y <= win->children[count]->y +
+					           win->children[count]->height)
+					{
+						win = win->children[count] ;
+						p.x -= win->x ;
+						p.y -= win->y ;
+					}
+				}
+
+				bev.x = p.x - win->margin ;
+				bev.y = p.y - win->margin ;
+
 				if( ((SHORT)HIWORD(event->wparam)) > 0)
 				{
 					bev.button = 4 ;
@@ -1991,17 +2072,25 @@ x_window_receive_event(
 					bev.button = 5 ;
 				}
 			}
-			else if( event->msg == WM_LBUTTONDOWN || event->msg == WM_LBUTTONUP)
+			else
 			{
-				bev.button = 1 ;
-			}
-			else if( event->msg == WM_RBUTTONDOWN || event->msg == WM_RBUTTONUP)
-			{
-				bev.button = 3 ;
-			}
-			else /* if( event->msg == WM_MBUTTONDOWN) || event->msg == WM_MBUTTONUP) */
-			{
-				bev.button = 2 ;
+				bev.x = LOWORD(event->lparam) - win->margin ;
+				bev.y = HIWORD(event->lparam) - win->margin ;
+
+				if( event->msg == WM_LBUTTONDOWN || event->msg == WM_LBUTTONUP)
+				{
+					bev.button = 1 ;
+				}
+				else if( event->msg == WM_MBUTTONDOWN ||
+					event->msg == WM_MBUTTONUP)
+				{
+					bev.button = 2 ;
+				}
+				else /* if( event->msg == WM_RBUTTONDOWN ||
+					event->msg == WM_RBUTTONUP) */
+				{
+					bev.button = 3 ;
+				}
 			}
 
 			if( event->msg == WM_MOUSEWHEEL || event->msg == WM_LBUTTONDOWN ||
@@ -2054,7 +2143,7 @@ x_window_receive_event(
 		return  1 ;
 
 	case  WM_MOUSEMOVE:
-		if( win->button_is_pressing)
+		if( win->button_is_pressing || win->pointer_motion)
 		{
 			XMotionEvent  mev ;
 
@@ -2064,18 +2153,25 @@ x_window_receive_event(
 			mev.y = HIWORD(event->lparam) - win->margin ;
 
 			mev.state = 0 ;
+
 			if( event->wparam & MK_LBUTTON)
 			{
 				mev.state |= Button1Mask ;
 			}
-			if( event->wparam & MK_RBUTTON)
+			if( event->wparam & MK_MBUTTON)
 			{
 				mev.state |= Button2Mask ;
 			}
-			if( event->wparam & MK_MBUTTON)
+			if( event->wparam & MK_RBUTTON)
 			{
 				mev.state |= Button3Mask ;
 			}
+
+			if( ! mev.state)
+			{
+				win->button_is_pressing = 0 ;
+			}
+			
 			if( event->wparam & MK_SHIFT)
 			{
 				mev.state |= ShiftMask ;
@@ -2085,20 +2181,38 @@ x_window_receive_event(
 				mev.state |= ControlMask ;
 			}
 
-			if( win->button_motion)
+			if( win->button_is_pressing)
 			{
-				(*win->button_motion)( win , &mev) ;
-			}
-			
-			/* following button motion ... */
-			win->prev_button_press_event.x = mev.x ;
-			win->prev_button_press_event.y = mev.y ;
-			win->prev_button_press_event.time = mev.time ;
+				if( win->button_motion)
+				{
+					(*win->button_motion)( win , &mev) ;
+				}
 
-		#if  0
-			kik_debug_printf( KIK_DEBUG_TAG " mouse motion... state %d x %d y %d\n",
-				mev.state, mev.x, mev.y) ;
-		#endif
+				/* following button motion ... */
+				win->prev_button_press_event.x = mev.x ;
+				win->prev_button_press_event.y = mev.y ;
+				win->prev_button_press_event.time = mev.time ;
+
+			#if  0
+				kik_debug_printf( KIK_DEBUG_TAG
+					" button motion... state %d x %d y %d\n" ,
+					mev.state , mev.x , mev.y) ;
+			#endif
+			}
+			/*
+			 * win->pointer_motion should be checked again here because
+			 * win->button_is_pressing was changed from 1 to 0 above.
+			 */
+			else if( win->pointer_motion)
+			{
+				(*win->pointer_motion)( win , &mev) ;
+
+			#if  0
+				kik_debug_printf( KIK_DEBUG_TAG
+					" pointer motion... state %d x %d y %d\n" ,
+					mev.state , mev.x , mev.y) ;
+			#endif
+			}
 		}
 		
 		return  1 ;
@@ -2217,6 +2331,9 @@ x_window_receive_event(
 							  total_height_inc(win) ) ;
 				}
 				
+				win->width = width - win->margin * 2 ;
+				win->height = height - win->margin * 2 ;
+
 				if( width_surplus > 0 || height_surplus > 0)
 				{
 					x_window_resize( win,
@@ -2226,9 +2343,6 @@ x_window_receive_event(
 				}
 				else
 				{
-					win->width = width - win->margin * 2 ;
-					win->height = height - win->margin * 2 ;
-
 					x_window_clear_all( win) ;
 
 					(*win->window_resized)( win) ;
@@ -2239,6 +2353,14 @@ x_window_receive_event(
 		}
 
 		return  1 ;
+
+	/* Not necessary */
+#if  0
+	case  WM_THEMECHANGED:
+		update_decorate_size( win) ;
+
+		return  1 ;
+#endif
 	}
 
 	return  -1 ;
@@ -2453,7 +2575,7 @@ x_window_draw_decsp_string(
 	u_int  len
 	)
 {
-	return  0 ;
+	return  x_window_draw_string( win , font , fg_color , x , y , str , len) ;
 }
 
 int
@@ -2468,46 +2590,8 @@ x_window_draw_decsp_image_string(
 	u_int  len
 	)
 {
-#ifndef  USE_WIN32GUI
-	if( font->decsp_font)
-	{
-		if( fg_color)
-		{
-			XSetForeground( win->display , win->ch_gc , fg_color->pixel) ;
-		}
-
-		if( bg_color)
-		{
-			XSetBackground( win->display , win->ch_gc , bg_color->pixel) ;
-			x_decsp_font_draw_image_string( font->decsp_font ,
-						  win->display , win->my_window , win->ch_gc ,
-						  x + win->margin , y + win->margin , str , len) ;
-		}
-		else
-		{
-			x_decsp_font_draw_string( font->decsp_font ,
-						  win->display , win->my_window , win->ch_gc ,
-						  x + win->margin , y + win->margin , str , len) ;
-		}
-		return  1 ;
-	}
-	else if( font->xfont)
-	{
-		if( bg_color)
-		{
-			return  x_window_draw_image_string( win , font , fg_color , bg_color ,
-					x , y , str , len) ;
-		}
-		else
-		{
-			return  x_window_draw_string( win , font , fg_color , x , y , str , len) ;
-		}
-	}
-	else
-#endif
-	{
-		return  0 ;
-	}
+	return  x_window_draw_image_string( win , font , fg_color , bg_color ,
+			x , y , str , len) ;
 }
 
 int
@@ -2711,13 +2795,13 @@ x_set_use_clipboard_selection(
 	int  use_it
 	)
 {
-	return  1 ;
+	return  0 ;
 }
 
 int
 x_is_using_clipboard_selection(void)
 {
-	return  1 ;
+	return  0 ;
 }
 
 int
@@ -2904,7 +2988,7 @@ x_set_window_name(
 	 * Convert name to current locale encoding or UTF16.
 	 * Use SetWindowTextW if UTF16_IME_CHAR is defined.
 	 */
-	SetWindowText( root->my_window , name) ;
+	SetWindowTextA( root->my_window , name) ;
 
 	return  1 ;
 }
